@@ -5,7 +5,7 @@
 
 	mtPaint is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 2 of the License, or
+	the Free Software Foundation; either version 3 of the License, or
 	(at your option) any later version.
 
 	mtPaint is distributed in the hope that it will be useful,
@@ -17,14 +17,7 @@
 	along with mtPaint in the file COPYING.
 */
 
-#include <gtk/gtk.h>
-
-
-
 #define MAX_CYC_SLOTS 100
-#define TOT_CYC_ITEMS 52
-// 52 = 2 items for start/finish, 50 for collection of layer #'s
-
 
 #define MAX_FRAME 99999
 
@@ -49,9 +42,5 @@ void ani_read_file( FILE *fp );		// Read data from layers file already opened
 void ani_write_file( FILE *fp );	// Write data to layers file already opened
 
 void ani_but_preview();			// Preview the animation
-void ani_set_key_frame(int key);	// Set key frame postions & cycles as per current layers
-
-void wild_space_change( char *in, char *out, int length );
-					// Copy string but replace " " with "\ " or "\" \"" - needed for filenames with spaces
 
 int gifsicle( char *command );	// Execute Gifsicle/Gifview
