@@ -5,7 +5,7 @@
 
 	mtPaint is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 2 of the License, or
+	the Free Software Foundation; either version 3 of the License, or
 	(at your option) any later version.
 
 	mtPaint is distributed in the hope that it will be useful,
@@ -17,17 +17,13 @@
 	along with mtPaint in the file COPYING.
 */
 
-#include <gdk/gdkkeysyms.h>
-#include <string.h>
-#include <math.h>
-
+#include "mygtk.h"
 #include "memory.h"
 #include "png.h"
 #include "layer.h"
 #include "mainwindow.h"
 #include "otherwindow.h"
 #include "canvas.h"
-#include "mygtk.h"
 #include "inifile.h"
 #include "global.h"
 #include "viewer.h"
@@ -40,9 +36,9 @@ int	layers_total = 0,		// Layers currently being used
 	layer_selected = 0,		// Layer currently selected in the layers window
 	layers_changed = 0;		// 0=Unchanged
 
-char	layers_filename[256];		// Current filename for layers file
-gboolean show_layers_main = FALSE,	// Show all layers in main window
-	layers_pastry_cut = FALSE;	// Pastry cut layers in view area (for animation previews)
+char layers_filename[PATHBUF];		// Current filename for layers file
+int	show_layers_main,		// Show all layers in main window
+	layers_pastry_cut;		// Pastry cut layers in view area (for animation previews)
 
 
 layer_node layer_table[MAX_LAYERS+1];	// Table of layer info
@@ -50,34 +46,42 @@ layer_node layer_table[MAX_LAYERS+1];	// Table of layer info
 
 void layers_init()
 {
-	sprintf( layers_filename, _("Untitled") );
-	sprintf( layer_table[0].name, _("Background") );
+	strncpy0(layer_table[0].name, _("Background"), LAYER_NAMELEN);
 	layer_table[0].visible = TRUE;
 	layer_table[0].use_trans = FALSE;
 	layer_table[0].x = 0;
 	layer_table[0].y = 0;
 	layer_table[0].trans = 0;
 	layer_table[0].opacity = 100;
-
-	show_layers_main = inifile_get_gboolean( "layermainToggle", FALSE );
 }
 
-void repaint_layer(int l)		// Repaint layer in view/main window
+/* Allocate layer image, its channels and undo stack
+ * !!! Must be followed by update_undo() after setting up image is done */
+static layer_image *alloc_layer(int w, int h, int bpp, int cmask, chanlist src)
 {
+	layer_image *lim;
+
+	lim = calloc(1, sizeof(layer_image));
+	if (!lim) return (NULL);
+	if (init_undo(&lim->image_.undo_, mem_undo_depth) &&
+		mem_alloc_image(&lim->image_, w, h, bpp, cmask, src))
+		return (lim);
+	free(lim->image_.undo_.items);
+	free(lim);
+	return (NULL);
+}
+
+static void repaint_layer(int l)	// Repaint layer in view/main window
+{
+	image_info *image;
 	int lx, ly, lw, lh;
 
 	lx = layer_table[l].x;
 	ly = layer_table[l].y;
-	if ( l != layer_selected )
-	{
-		lw = layer_table[l].image->mem_width;
-		lh = layer_table[l].image->mem_height;
-	}
-	else
-	{
-		lw = mem_width;
-		lh = mem_height;
-	}
+	image = l == layer_selected ? &mem_image :
+		&layer_table[l].image->image_;
+	lw = image->width;
+	lh = image->height;
 	if (layer_selected)
 	{
 		lx -= layer_table[layer_selected].x;
@@ -100,7 +104,7 @@ typedef struct {
 } layer_item;
 
 static GtkWidget *layer_list, *entry_layer_name,
-	*layer_buttons[10], *layer_spin, *layer_slider,
+	*layer_tools[TOTAL_ICONS_LAYER], *layer_spin, *layer_slider,
 	*layer_label_position, *layer_trans_toggle, *layer_show_toggle;
 static layer_item layer_list_data[MAX_LAYERS + 1];
 
@@ -110,15 +114,16 @@ gboolean layers_initialized;		// Indicates if initializing is complete
 
 static void layers_update_titlebar()		// Update filename in titlebar
 {
-	char txt[300], txt2[600], *extra = "-";
+	char txt[300], txt2[PATHTXT], *extra = "-";
 
 	if ( layers_window == NULL ) return;		// Don't bother if window is not showing
 
-	gtkuncpy(txt2, layers_filename, 512);
+	gtkuncpy(txt2, layers_filename, PATHTXT);
 
 	if ( layers_changed == 1 ) extra = _("(Modified)");
 
-	snprintf( txt, 290, "%s %s %s", _("Layers"), extra, txt2 );
+	snprintf( txt, 290, "%s %s %s", _("Layers"), extra, txt2[0] ? txt2 :
+		_("Untitled"));
 
 	gtk_window_set_title (GTK_WINDOW (layers_window), txt );
 }
@@ -146,76 +151,17 @@ static void layer_copy_from_main( int l )	// Copy info from main image to layer
 {
 	layer_image *lp = layer_table[l].image;
 
-	memcpy(lp->mem_filename, mem_filename, 256);
-	memcpy(lp->mem_pal, mem_pal, sizeof(mem_pal));
-	memcpy(lp->mem_prot_RGB, mem_prot_RGB, sizeof(mem_prot_RGB));
-	memcpy(lp->mem_prot_mask, mem_prot_mask, sizeof(mem_prot_mask));
-	memcpy(lp->mem_undo_im_, mem_undo_im_, sizeof(undo_item) * MAX_UNDO);
-	memcpy(lp->mem_img, mem_img, sizeof(chanlist));
-
-	lp->mem_channel		= mem_channel;
-	lp->mem_img_bpp		= mem_img_bpp;
-	lp->mem_changed		= mem_changed;
-	lp->mem_width		= mem_width;
-	lp->mem_height		= mem_height;
-	lp->mem_ics		= mem_ics;
-	lp->mem_icx		= mem_icx;
-	lp->mem_icy		= mem_icy;
-
-	lp->mem_undo_pointer	= mem_undo_pointer;
-	lp->mem_undo_done	= mem_undo_done;
-	lp->mem_undo_redo	= mem_undo_redo;
-
-	lp->mem_cols		= mem_cols;
-	lp->tool_pat		= tool_pat;
-	lp->mem_col_A		= mem_col_A;
-	lp->mem_col_B		= mem_col_B;
-	lp->mem_col_A24		= mem_col_A24;
-	lp->mem_col_B24		= mem_col_B24;
-
-	lp->mem_xpm_trans	= mem_xpm_trans;
-	lp->mem_xbm_hot_x	= mem_xbm_hot_x;
-	lp->mem_xbm_hot_y	= mem_xbm_hot_y;
-
-	lp->mem_prot		= mem_prot;
+	lp->image_ = mem_image;
+	lp->state_ = mem_state;
+	lp->image_.size = 0; // Invalidate
 }
 
 static void layer_copy_to_main( int l )		// Copy info from layer to main image
 {
 	layer_image *lp = layer_table[l].image;
 
-	memcpy(mem_filename, lp->mem_filename, 256);
-	memcpy(mem_pal, lp->mem_pal, sizeof(mem_pal));
-	memcpy(mem_prot_RGB, lp->mem_prot_RGB, sizeof(mem_prot_RGB));
-	memcpy(mem_prot_mask, lp->mem_prot_mask, sizeof(mem_prot_mask));
-	memcpy(mem_undo_im_, lp->mem_undo_im_, sizeof(undo_item) * MAX_UNDO);
-	memcpy(mem_img, lp->mem_img, sizeof(chanlist));
-
-	mem_channel	= lp->mem_channel;
-	mem_img_bpp	= lp->mem_img_bpp;
-	mem_changed	= lp->mem_changed;
-	mem_width	= lp->mem_width;
-	mem_height	= lp->mem_height;
-	mem_ics 	= lp->mem_ics;
-	mem_icx 	= lp->mem_icx;
-	mem_icy 	= lp->mem_icy;
-
-	mem_undo_pointer = lp->mem_undo_pointer;
-	mem_undo_done	= lp->mem_undo_done;
-	mem_undo_redo	= lp->mem_undo_redo;
-
-	mem_cols	= lp->mem_cols;
-	tool_pat	= lp->tool_pat;
-	mem_col_A	= lp->mem_col_A;
-	mem_col_B	= lp->mem_col_B;
-	mem_col_A24	= lp->mem_col_A24;
-	mem_col_B24	= lp->mem_col_B24;
-
-	mem_xpm_trans	= lp->mem_xpm_trans;
-	mem_xbm_hot_x	= lp->mem_xbm_hot_x;
-	mem_xbm_hot_y	= lp->mem_xbm_hot_y;
-
-	mem_prot	= lp->mem_prot;
+	mem_image = lp->image_;
+	mem_state = lp->state_;
 }
 
 static void shift_layer(int val)
@@ -241,19 +187,19 @@ static void shift_layer(int val)
 	gtk_list_select_child( GTK_LIST(layer_list), layer_list_data[layer_selected].item );
 	layers_notify_changed();
 	if ( layer_selected == layers_total )
-		gtk_widget_set_sensitive( layer_buttons[1], FALSE );	// Raise button
+		gtk_widget_set_sensitive(layer_tools[LTB_RAISE], FALSE);
 	if ( layer_selected == 0 )
-		gtk_widget_set_sensitive( layer_buttons[2], FALSE );	// Lower button
+		gtk_widget_set_sensitive(layer_tools[LTB_LOWER], FALSE);
 
-	if ( val==1 ) gtk_widget_set_sensitive( layer_buttons[2], TRUE );	// Lower button
-	if ( val==-1 ) gtk_widget_set_sensitive( layer_buttons[1], TRUE );	// Raise button
+	if ( val==1 ) gtk_widget_set_sensitive(layer_tools[LTB_LOWER], TRUE);
+	if ( val==-1 ) gtk_widget_set_sensitive(layer_tools[LTB_RAISE], TRUE);
 
 	update_cols();				// Update status bar info
 
-	if ( layer_selected == 0 || (layer_selected-val) == 0 )
+	if ((layer_selected == 0) || (layer_selected == val))
 	{
-		if ( vw_drawing != NULL ) gtk_widget_queue_draw( vw_drawing );	// Update Whole window
-		if ( show_layers_main ) gtk_widget_queue_draw(drawing_canvas);
+		if (view_showing) gtk_widget_queue_draw(vw_drawing);
+		if (show_layers_main) gtk_widget_queue_draw(drawing_canvas);
 	}
 	else
 	{
@@ -274,12 +220,8 @@ static gint layer_press_lower()
 }
 
 
-void layer_new_chores( int l, int w, int h, int type, int cols,
-			chanlist temp_img, layer_image *lim )
+static void layer_new_chores(int l, layer_image *lim)
 {
-	int bpp = type, i, j;
-	png_color temp_pal[256];
-
 	if ( marq_status > MARQUEE_NONE )	// If we are selecting or pasting - lose it!
 	{
 		pressed_select_none(NULL, NULL);
@@ -287,9 +229,7 @@ void layer_new_chores( int l, int w, int h, int type, int cols,
 			icon_buttons[DEFAULT_TOOL_ICON]), TRUE );
 	}
 
-	if ( type == 2 ) bpp = 1;	// Type 2 = greyscale indexed
-
-	layer_table[l].image = lim;			// Image info memory pointer
+	layer_table[l].image = lim;		// Image info memory pointer
 
 	layer_table[l].name[0] = 0;
 	layer_table[l].x = 0;
@@ -299,73 +239,19 @@ void layer_new_chores( int l, int w, int h, int type, int cols,
 	layer_table[l].visible = TRUE;
 	layer_table[l].use_trans = FALSE;
 
-	j = w * h * bpp;
-	i = 0;
-#ifdef U_GUADALINEX
-	if ( bpp == 3 ) i = 255;
-#endif
-	memset(temp_img[CHN_IMAGE], i, j);
+	lim->state_.channel = lim->image_.img[mem_channel] ? mem_channel : CHN_IMAGE;
 
-	if ( type == 2 )	// Greyscale
-	{
-		mem_pal_copy( temp_pal, mem_pal );		// Save current
-#ifdef U_GUADALINEX
-		mem_scale_pal( 0, 255,255,255, cols-1, 0,0,0 );
-#else
-		mem_scale_pal( 0, 0,0,0, cols-1, 255,255,255 );
-#endif
-		mem_pal_copy( lim->mem_pal, mem_pal );
-		mem_pal_copy( temp_pal, mem_pal );		// Restore
-	}
-	else mem_pal_copy( lim->mem_pal, mem_pal_def );
+	lim->state_.col_[0] = 1;
+	lim->state_.col_[1] = 0;
+	lim->state_.col_24[0] = lim->image_.pal[1];
+	lim->state_.col_24[1] = lim->image_.pal[0];
 
-	sprintf( lim->mem_filename, _("Untitled") );
-
-	memset(lim->mem_img, 0, sizeof(chanlist));
-	memcpy(lim->mem_img, temp_img, sizeof(chanlist));
-	lim->mem_channel = temp_img[mem_channel] ? mem_channel : CHN_IMAGE;
-	lim->mem_img_bpp = bpp;
-	lim->mem_changed = 0;
-	lim->mem_width = w;
-	lim->mem_height = h;
-	lim->mem_ics = 0;
-	lim->mem_icx = 0;
-	lim->mem_icy = 0;
-
-	lim->mem_undo_pointer = 0;
-	lim->mem_undo_done = 0;
-	lim->mem_undo_redo = 0;
-
-	memset(lim->mem_undo_im_, 0, sizeof(undo_item) * MAX_UNDO);
-
-	memcpy(lim->mem_undo_im_[0].img, temp_img, sizeof(chanlist));
-	lim->mem_undo_im_[0].cols = cols;
-	lim->mem_undo_im_[0].bpp = bpp;
-	lim->mem_undo_im_[0].width = w;
-	lim->mem_undo_im_[0].height = h;
-	mem_pal_copy(lim->mem_undo_im_[0].pal, lim->mem_pal);
-
-	lim->mem_cols = cols;
-	lim->tool_pat = 0;
-	lim->mem_col_A = 1;
-	lim->mem_col_B = 0;
-	lim->mem_col_A24 = lim->mem_pal[1];
-	lim->mem_col_B24 = lim->mem_pal[0];
-	for ( i=0; i<256; i++ )
-	{
-		lim->mem_prot_RGB[i] = 0;
-		lim->mem_prot_mask[i] = 0;
-	}
-	lim->mem_prot = 0;
-
-	lim->mem_xpm_trans = -1;
-	lim->mem_xbm_hot_x = -1;
-	lim->mem_xbm_hot_y = -1;
-
-	lim->ani_pos[0][0] = 0;
+	lim->state_.xpm_trans = -1;
+	lim->state_.xbm_hot_x = -1;
+	lim->state_.xbm_hot_y = -1;
 }
 
-void layer_new_chores2( int l )
+static void layer_new_chores2( int l )
 {
 	if ( layers_window != NULL )
 	{
@@ -377,16 +263,13 @@ void layer_new_chores2( int l )
 			layer_table[l].visible);
 
 		gtk_list_select_child( GTK_LIST(layer_list), layer_list_data[l].item );
-		gtk_widget_set_sensitive( layer_buttons[1], FALSE );	// Raise button
-		if ( l == 1 )
-			gtk_widget_set_sensitive( layer_buttons[2], FALSE );	// Lower button
-		else
-			gtk_widget_set_sensitive( layer_buttons[2], TRUE );	// Lower button
+		gtk_widget_set_sensitive(layer_tools[LTB_RAISE], FALSE);
+		gtk_widget_set_sensitive(layer_tools[LTB_LOWER], l != 1);
 
-		if ( l == MAX_LAYERS )			// Hide new/duplicate if we have max layers
+		if ( l == MAX_LAYERS )		// Hide new/duplicate if we have max layers
 		{
-			gtk_widget_set_sensitive( layer_buttons[3], FALSE );
-			gtk_widget_set_sensitive( layer_buttons[4], FALSE );
+			gtk_widget_set_sensitive(layer_tools[LTB_NEW], FALSE);
+			gtk_widget_set_sensitive(layer_tools[LTB_DUP], FALSE);
 		}
 	}
 
@@ -396,10 +279,8 @@ void layer_new_chores2( int l )
 
 void layer_new( int w, int h, int type, int cols, int cmask )	// Types 1=indexed, 2=grey, 3=RGB
 {
-	int i, j = w * h, bpp=type;
-	chanlist temp_img;
+	int bpp;
 	layer_image *lim;
-	unsigned char *rgb;
 
 	if ( layers_total>=MAX_LAYERS ) return;
 
@@ -409,34 +290,30 @@ void layer_new( int w, int h, int type, int cols, int cmask )	// Types 1=indexed
 	}
 	layer_copy_from_main( layer_selected );
 
-	if ( type == 2 ) bpp = 1;	// Type 2 = greyscale indexed
+	bpp = type == 2 ? 1 : type;	// Type 2 = greyscale indexed
 
-	lim = malloc(sizeof(layer_image));
+	lim = alloc_layer(w, h, bpp, cmask, NULL);
 	if (!lim)
 	{
 		memory_errors(1);
 		return;
 	}
-	memset(temp_img, 0, sizeof(chanlist));
-	rgb = temp_img[CHN_IMAGE] = calloc(1, j * bpp);
-	for (i = CHN_ALPHA; rgb && (cmask > CMASK_FOR(i)); i++)
+
+	lim->image_.cols = cols;
+	mem_pal_copy(lim->image_.pal, mem_pal_def); // Default
+	if ( type == 2 )	// Greyscale
 	{
-		if (!(cmask & CMASK_FOR(i))) continue;
-		rgb = temp_img[i] = calloc(1, j);
-	}
-	if (!rgb)	// Not enough memory
-	{
-		free(lim);
-		for (i = 0; i < NUM_CHANNELS; i++)
-			free(temp_img[i]);
-		memory_errors(1);
-		return;
+#ifdef U_GUADALINEX
+		mem_scale_pal(lim->image_.pal, 0, 255,255,255, cols - 1, 0,0,0);
+#else
+		mem_scale_pal(lim->image_.pal, 0, 0,0,0, cols - 1, 255,255,255);
+#endif
 	}
+	update_undo(&lim->image_);
 
 	layers_total++;
-
-	layer_new_chores( layers_total, w, h, type, cols, temp_img, lim );
-	layer_new_chores2( layers_total );
+	layer_new_chores(layers_total, lim);
+	layer_new_chores2(layers_total);
 	layer_selected = layers_total;
 
 	if ( layers_total == 1 ) ani_init();		// Start with fresh animation data if new
@@ -451,9 +328,8 @@ static gint layer_press_new()
 
 static gint layer_press_duplicate()
 {
-	layer_image *lim;
-	chanlist temp_img;
-	int w = mem_width, h = mem_height, bpp = mem_img_bpp, cols = mem_cols, i, j;
+	layer_image *lim, *ls;
+	int w = mem_width, h = mem_height, bpp = mem_img_bpp;
 
 	if ( layers_total>=MAX_LAYERS ) return FALSE;
 
@@ -464,53 +340,25 @@ static gint layer_press_duplicate()
 	if ( layers_total == 0 ) layer_table[0].image = malloc( sizeof(layer_image) );
 	layer_copy_from_main( layer_selected );
 
-	lim = malloc( sizeof(layer_image) );
+	lim = alloc_layer(w, h, bpp, 0, mem_img);
 	if (!lim)
 	{
 		memory_errors(1);
 		goto end;
 	}
-	memset(temp_img, 0, sizeof(chanlist));
-	j = mem_width * mem_height;
-	for (i = 0; i < NUM_CHANNELS; i++)
-	{
-		if (!mem_img[i]) continue;
-		if ((temp_img[i] = malloc(j * BPP(i)))) continue;
-		free(lim);
-		for (j = 0; j < i; j++) free(temp_img[j]);
-		memory_errors(1);
-		goto end;
-	}
 	layers_total++;
 
-	layer_new_chores( layers_total, w, h, bpp, cols, temp_img, lim);
-
 	layer_table[layers_total] = layer_table[layer_selected];		// Copy layer info
 	layer_table[layers_total].image = lim;
+	ls = layer_table[layer_selected].image;
 
-	for ( i=0; i<256; i++ )
-	{
-		layer_table[layers_total].image->mem_pal[i] =
-			layer_table[layer_selected].image->mem_pal[i];
-		layer_table[layers_total].image->mem_prot_RGB[i] =
-			layer_table[layer_selected].image->mem_prot_RGB[i];
-		layer_table[layers_total].image->mem_prot_mask[i] =
-			layer_table[layer_selected].image->mem_prot_mask[i];
-	}
+	lim->state_ = ls->state_;
+	mem_pal_copy(lim->image_.pal, ls->image_.pal);
+	lim->image_.cols = ls->image_.cols;
+	update_undo(&lim->image_);
 
-	j = mem_width * mem_height;
-	for (i = 0; i < NUM_CHANNELS; i++)
-	{
-		if (!mem_img[i]) continue;
-		memcpy(temp_img[i], mem_img[i], j * BPP(i));
-	}
-	sprintf( layer_table[layers_total].image->mem_filename,
-		layer_table[layer_selected].image->mem_filename );
-
-	for ( i=0; i<MAX_POS_SLOTS; i++ ) for ( j=0; j<5; j++ )
-		layer_table[layers_total].image->ani_pos[i][j] =
-			layer_table[layer_selected].image->ani_pos[i][j];
-				// Copy across position data
+	// Copy across position data
+	memcpy(lim->ani_pos, ls->ani_pos, sizeof(lim->ani_pos));
 
 	layer_new_chores2( layers_total );
 	layer_selected = layers_total;
@@ -528,10 +376,8 @@ static void layer_delete(int item)
 	layer_image *lp = layer_table[item].image;
 	int i;
 
-	for (i=0; i<MAX_UNDO; i++)		// Release old UNDO images
-		undo_free_x(&lp->mem_undo_im_[i]);
-
-	free( layer_table[item].image );
+	mem_free_image(&lp->image_, FREE_ALL);
+	free(lp);
 
 	if (item < layers_total)	// If deleted item is not at the end shuffle rest down
 		for ( i=item; i<layers_total; i++ )
@@ -571,17 +417,18 @@ static void layer_refresh_list()
 		}
 	}
 	if ( layer_selected == layers_total )
-		gtk_widget_set_sensitive( layer_buttons[1], FALSE );	// Raise button
-	gtk_widget_set_sensitive( layer_buttons[3], TRUE );		// New
-	gtk_widget_set_sensitive( layer_buttons[4], TRUE );		// Duplicate
+		gtk_widget_set_sensitive(layer_tools[LTB_RAISE], FALSE);
+	gtk_widget_set_sensitive(layer_tools[LTB_NEW], TRUE);
+	gtk_widget_set_sensitive(layer_tools[LTB_DUP], TRUE);
 }
 
 static gint layer_press_delete()
 {
-	char txt[128];
+	char txt[256];
 	int i, to_go = layer_selected;
 
-	sprintf(txt, _("Do you really want to delete layer %i (%s) ?"), layer_selected, layer_table[layer_selected].name );
+	snprintf(txt, 256, _("Do you really want to delete layer %i (%s) ?"),
+		layer_selected, layer_table[layer_selected].name );
 
 	i = alert_box( _("Warning"), txt, _("No"), _("Yes"), NULL );
 
@@ -614,8 +461,10 @@ static void layer_show_position()
 
 static gint layer_press_centre()
 {
-	layer_table[layer_selected].x = layer_table[0].image->mem_width / 2 - mem_width / 2;
-	layer_table[layer_selected].y = layer_table[0].image->mem_height / 2 - mem_height / 2;
+	layer_table[layer_selected].x = layer_table[0].image->image_.width / 2 -
+		mem_width / 2;
+	layer_table[layer_selected].y = layer_table[0].image->image_.height / 2 -
+		mem_height / 2;
 	layer_show_position();
 	layers_notify_changed();
 	update_all_views();
@@ -626,14 +475,11 @@ static gint layer_press_centre()
 int layers_unsaved_tot()			// Return number of layers with no filenames
 {
 	int j = 0, k;
-	char *t;
 
 	for ( k=0; k<=layers_total; k++ )	// Check each layer for proper filename
 	{
-		if ( k == layer_selected ) t = mem_filename;
-		else t = layer_table[k].image->mem_filename;
-
-		if ( strcmp( t, _("Untitled") ) == 0 ) j++;
+		j += !(k == layer_selected ? mem_filename[0] :
+			layer_table[k].image->state_.filename[0]);
 	}
 
 	return j;
@@ -641,20 +487,15 @@ int layers_unsaved_tot()			// Return number of layers with no filenames
 
 int layers_changed_tot()			// Return number of layers with changes
 {
-	int j = 0, k;
+	image_state *state;
+	int j, k;
 
-	for ( k=0; k<=layers_total; k++ )	// Check each layer for mem_changed
+	for (j = k =0; k <= layers_total; k++)	// Check each layer for mem_changed
 	{
-		if ( k == layer_selected )
-		{
-			j = j + mem_changed;
-			if ( strcmp( mem_filename, _("Untitled") ) == 0 ) j++;
-		}
-		else
-		{
-			j = j + layer_table[k].image->mem_changed;
-			if ( strcmp( layer_table[k].image->mem_filename, _("Untitled") ) == 0 ) j++;
-		}
+		state = k == layer_selected ? &mem_state :
+			&layer_table[k].image->state_;
+		j += state->changed;
+		j += !state->filename[0];
 	}
 
 	return j;
@@ -673,9 +514,9 @@ int check_layers_for_changes()			// 1=STOP, 2=IGNORE, 10=ESCAPE, -10=NOT CHECKED
 	return i;
 }
 
-void layer_update_filename( char *name )
+static void layer_update_filename( char *name )
 {
-	strncpy(layers_filename, name, 250);
+	strncpy(layers_filename, name, PATHBUF);
 	layers_changed = 1;		// Forces update of titlebar
 	layers_notify_unchanged();
 }
@@ -690,9 +531,9 @@ void string_chop( char *txt )
 
 int read_file_num(FILE *fp, char *txt)
 {
-	int i = get_next_line(txt, 32, fp);
+	int i;
 
-	if ( i<0 || i>30 ) return -987654321;
+	if (!fgets(txt, 32, fp)) return -987654321;
 	sscanf(txt, "%i", &i);
 
 	return i;
@@ -702,21 +543,19 @@ static void layers_remove_all(); /* Forward declaration */
 
 int load_layers( char *file_name )
 {
-	char tin[300], load_prefix[300], load_name[300], *c;
+	char tin[300], load_name[PATHBUF], *c;
 	layer_image *lim2;
 	int i, j, k, layers_to_read = -1, layer_file_version = -1, lfail = 0;
+	int lplen = 0;
 	FILE *fp;
 
-	strncpy( load_prefix, file_name, 256 );
-	c = strrchr( load_prefix, DIR_SEP );
-	if ( c!=NULL ) c[1]=0;
-	else load_prefix[0]=0;
+	c = strrchr(file_name, DIR_SEP);
+	if (c) lplen = c - file_name + 1;
 
 		// Try to save text file, return -1 if failure
 	if ((fp = fopen(file_name, "r")) == NULL) goto fail;
 
-	i = get_next_line(tin, 32, fp);
-	if ( i<0 || i>30 ) goto fail2;
+	if (!fgets(tin, 32, fp)) goto fail2;
 
 	string_chop( tin );
 	if ( strcmp( tin, LAYERS_HEADER ) != 0 ) goto fail2;		// Bad header
@@ -734,13 +573,13 @@ int load_layers( char *file_name )
 	for ( i=0; i<=layers_to_read; i++ )
 	{
 		// Read filename, strip end chars & try to load (if name length > 0)
-		j = get_next_line(tin, 256, fp);
+		fgets(tin, 256, fp);
 		string_chop(tin);
-		snprintf(load_name, 260, "%s%s", load_prefix, tin);
+		snprintf(load_name, PATHBUF, "%.*s%s", lplen, file_name, tin);
 		k = 1;
 		j = detect_image_format(load_name);
 		if ((j > 0) && (j != FT_NONE) && (j != FT_LAYERS1))
-			k = load_image(load_name, FS_PNG_LOAD, j) != 1;
+			k = load_image(load_name, FS_LAYER_LOAD, j) != 1;
 
 		if (k) /* Failure - skip this layer */
 		{
@@ -764,9 +603,9 @@ int load_layers( char *file_name )
 		init_istate(); /* Update image variables after load */
 		layer_copy_from_main( layers_total );
 
-		j = get_next_line(tin, 256, fp);
+		fgets(tin, 256, fp);
 		string_chop(tin);
-		strncpy(layer_table[layers_total].name, tin, 32); // Layer text name
+		strncpy0(layer_table[layers_total].name, tin, LAYER_NAMELEN);
 
 		k = read_file_num(fp, tin);
 		layer_table[layers_total].visible = k > 0;
@@ -785,18 +624,21 @@ int load_layers( char *file_name )
 
 		layers_total++;
 
+// !!! A brittle hack - have to find other way to stop mem_free_image() in loader
 		// Bogus 1x1 image used
 		mem_width = 1;
 		mem_height = 1;
 		memset(mem_img, 0, sizeof(chanlist));
-		memset(&mem_undo_im_[0].img, 0, sizeof(chanlist));
+		init_undo(&mem_image.undo_, mem_undo_depth);
 		mem_undo_im_[0].img[CHN_IMAGE] = mem_img[CHN_IMAGE] = malloc(3);
 	}
 	if ( layers_total>0 )
 	{
 		layers_total--;
 
-		mem_clear();	// Lose excess bogus memory
+		/* Free unused mem_image */
+		mem_free_image(&mem_image, FREE_ALL);
+
 		layer_copy_to_main(layers_total);
 		if ( layers_total == 0 )
 		{	// You will need to free the memory holding first layer if just 1 was loaded
@@ -810,7 +652,9 @@ int load_layers( char *file_name )
 	}
 	else layer_refresh_list();
 
-	ani_read_file(fp);		// Read in animation data
+	/* Read in animation data - only if all layers loaded OK
+	 * (to do otherwise is likely to result in SIGSEGV) */
+	if (!lfail) ani_read_file(fp);
 
 	fclose(fp);
 	layer_update_filename( file_name );
@@ -830,35 +674,29 @@ fail:
 	return -1;
 }
 
-void parse_filename( char *dest, char *prefix, char *file )
-{	// Convert absolute filename 'file' into one relative to prefix
-	int i = 0, j = strlen(prefix), k;
+/* Convert absolute filename 'file' into one relative to prefix */
+static void parse_filename(char *dest, char *prefix, char *file, int len)
+{
+	int i, k;
 
-	while ( i<j && prefix[i] == file[i] ) i++;	// # of chars that match at start
+	/* # of chars that match at start */
+	for (i = 0; (i < len) && (prefix[i] == file[i]); i++);
 
-	if ( i>0 )
+	if (!i || (i == len)) /* Complete match, or no match at all */
+		strncpy(dest, file + i, PATHBUF);
+	else	/* Partial match */
 	{
-		if ( i==j )				// File is at prefix level or below
-			strncpy( dest, file+i, 256 );
-		else					// File is below prefix level
-		{
-			dest[0]=0;
-			k = i;
-			while ( k<j )
-			{
-				if ( prefix[k] == DIR_SEP ) strncat( dest, "../", 256 );
-				k++;
-			} // Count number of DIR_SEP encountered on and after point i in 'prefix', add a '../' for each found
-			k = i;
-			while ( k>-1 && file[k]!=DIR_SEP )
-			{
-				k--;
-			} // nip backwards on 'file' from i to previous DIR_SEP or beginning and ..
-
-			strncat( dest, file+k+1, 256 );	// ... add rest of 'file'
-		}
+		dest[0] = 0;
+		/* Count number of DIR_SEP encountered on and after point i in
+		 * 'prefix', add a '../' for each found */
+		for (k = i; k < len; k++)
+			if (prefix[k] == DIR_SEP) strnncat( dest, "../", PATHBUF);
+		/* nip backwards on 'file' from i to previous DIR_SEP or
+		 * beginning and ... */
+		for (k = i; (k >= 0) && (file[k] != DIR_SEP); k--);
+		/* ... add rest of 'file' */
+		strnncat(dest, file + k + 1, PATHBUF);
 	}
-	else	strncpy( dest, file, 256 );		// Prefix not in file at all, so copy all
 }
 
 void layer_press_save_composite()		// Create, save, free the composite image
@@ -868,19 +706,13 @@ void layer_press_save_composite()		// Create, save, free the composite image
 
 int layer_save_composite(char *fname, ls_settings *settings)
 {
+	image_info *image;
 	unsigned char *layer_rgb;
 	int w, h, res=0;
 
-	if (layer_selected)
-	{
-		w = layer_table[0].image->mem_width;
-		h = layer_table[0].image->mem_height;
-	}
-	else
-	{
-		w = mem_width;
-		h = mem_height;
-	}
+	image = layer_selected ? &layer_table[0].image->image_ : &mem_image;
+	w = image->width;
+	h = image->height;
 	layer_rgb = malloc(w * h * 3);
 	if (layer_rgb)
 	{
@@ -891,10 +723,10 @@ int layer_save_composite(char *fname, ls_settings *settings)
 		settings->bpp = 3;
 		if (layer_selected) /* Set up background transparency */
 		{
-			res = layer_table[0].image->mem_xpm_trans;
+			res = layer_table[0].image->state_.xpm_trans;
 			settings->xpm_trans = res;
 			settings->rgb_trans = res < 0 ? -1 :
-				PNG_2_INT(layer_table[0].image->mem_pal[res]);
+				PNG_2_INT(layer_table[0].image->image_.pal[res]);
 
 		}
 		res = save_image(fname, settings);
@@ -907,14 +739,12 @@ int layer_save_composite(char *fname, ls_settings *settings)
 
 int save_layers( char *file_name )
 {
-	char mess[300], comp_name[300], save_prefix[300], *c;
-	int i;
+	char comp_name[PATHBUF], *c, *msg;
+	int i, l = 0;
 	FILE *fp;
 
-	strncpy( save_prefix, file_name, 256 );
-	c = strrchr( save_prefix, DIR_SEP );
-	if ( c!=NULL ) c[1]=0;
-	else save_prefix[0]=0;
+	c = strrchr(file_name, DIR_SEP);
+	if (c) l = c - file_name + 1;
 
 		// Try to save text file, return -1 if failure
 	if ((fp = fopen(file_name, "w")) == NULL) goto fail;
@@ -922,14 +752,9 @@ int save_layers( char *file_name )
 	fprintf( fp, "%s\n%i\n%i\n", LAYERS_HEADER, LAYERS_VERSION, layers_total );
 	for ( i=0; i<=layers_total; i++ )
 	{
-		if ( layer_selected == i )
-		{
-			parse_filename( comp_name, save_prefix, mem_filename );
-		}
-		else
-		{
-			parse_filename( comp_name, save_prefix, layer_table[i].image->mem_filename );
-		}
+		c = layer_selected == i ? mem_filename :
+			layer_table[i].image->state_.filename;
+		parse_filename(comp_name, file_name, c, l);
 		fprintf( fp, "%s\n", comp_name );
 
 		fprintf( fp, "%s\n%i\n%i\n%i\n%i\n%i\n%i\n",
@@ -946,8 +771,11 @@ int save_layers( char *file_name )
 
 	return 1;		// Success
 fail:
-	snprintf(mess, 260, _("Unable to save file: %s"), layers_filename);
-	alert_box( _("Error"), mess, _("OK"), NULL, NULL );
+	c = gtkuncpy(NULL, layers_filename, 0);
+	msg = g_strdup_printf(_("Unable to save file: %s"), c);
+	alert_box(_("Error"), msg, _("OK"), NULL, NULL);
+	g_free(msg);
+	g_free(c);
 
 	return -1;
 }
@@ -973,10 +801,7 @@ void layer_press_save_as()
 
 void layer_press_save()
 {
-	if ( strcmp( layers_filename, _("Untitled") ) == 0 )
-	{
-		layer_press_save_as();
-	}
+	if (!layers_filename[0]) layer_press_save_as();
 	else
 	{
 		check_layers_all_saved();
@@ -988,19 +813,16 @@ static void update_main_with_new_layer()
 {
 	int w, h;
 
+
+	update_titlebar();
+	update_menus();
+
+	init_pal();		// Update Palette, pattern & mask area + widgets
+
 	canvas_size(&w, &h);
 	gtk_widget_set_usize(drawing_canvas, w, h);
 	vw_focus_view();
 	update_all_views();
-
-	init_pal();		// Update Palette, pattern & mask area + widgets
-	gtk_widget_queue_draw(drawing_col_prev);
-
-	update_titlebar();
-	update_menus();
-	if ((tool_type == TOOL_SMUDGE) && (MEM_BPP == 1))
-		gtk_toggle_button_set_active(
-			GTK_TOGGLE_BUTTON(icon_buttons[DEFAULT_TOOL_ICON]), TRUE );
 }
 
 static void layers_remove_all()
@@ -1030,7 +852,7 @@ static void layers_remove_all()
 		layer_delete(i);
 	}
 	layer_refresh_list();
-	sprintf(layers_filename, _("Untitled"));
+	layers_filename[0] = 0;
 	layers_notify_unchanged();
 
 	if ( layers_window ) gtk_widget_set_sensitive( layers_window, TRUE);
@@ -1069,17 +891,17 @@ static gint layer_tog_visible( GtkWidget *widget, GdkEvent *event, gpointer data
 static gint layer_main_toggled( GtkWidget *widget, GdkEvent *event, gpointer data )
 {
 	show_layers_main = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(layer_show_toggle) );
-	inifile_set_gboolean( "layermainToggle", show_layers_main );
 	gtk_widget_queue_draw(drawing_canvas);
 
 	return FALSE;
 }
 
-static gint layer_inputs_changed()
+static void layer_inputs_changed()
 {
+	const char *nname;
 	gboolean txt_changed = FALSE;
 
-	if ( !layers_initialized ) return FALSE;
+	if (!layers_initialized) return;
 
 	layers_notify_changed();
 
@@ -1087,11 +909,11 @@ static gint layer_inputs_changed()
 			gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(layer_spin) );
 	layer_table[layer_selected].opacity = mt_spinslide_get_value(layer_slider);
 
-	if ( strncmp( layer_table[layer_selected].name,
-		gtk_entry_get_text( GTK_ENTRY(entry_layer_name) ), 35) ) txt_changed = TRUE;
+	nname = gtk_entry_get_text(GTK_ENTRY(entry_layer_name));
+	if (strncmp(layer_table[layer_selected].name, nname, LAYER_NAMELEN - 1))
+		txt_changed = TRUE;
 
-	strncpy( layer_table[layer_selected].name,
-		gtk_entry_get_text( GTK_ENTRY(entry_layer_name) ), 35);
+	strncpy0(layer_table[layer_selected].name, nname, LAYER_NAMELEN);
 	gtk_label_set_text( GTK_LABEL(layer_list_data[layer_selected].name),
 		layer_table[layer_selected].name );
 	layer_table[layer_selected].use_trans = gtk_toggle_button_get_active(
@@ -1099,8 +921,6 @@ static gint layer_inputs_changed()
 
 	if ( !txt_changed ) repaint_layer( layer_selected );
 		// Update layer image if not just changing text
-
-	return FALSE;
 }
 
 void layer_choose( int l )				// Select a new layer from the list
@@ -1154,10 +974,10 @@ static gint layer_select( GtkList *list, GtkWidget *widget, gpointer user_data )
 		gtk_entry_set_text( GTK_ENTRY(entry_layer_name), layer_table[j].name );
 		if ( j==0 )		// Background layer selected
 		{
-			gtk_widget_set_sensitive( layer_buttons[1], layers_total > 0);	// Raise button
-			gtk_widget_set_sensitive( layer_buttons[2], FALSE );	// Lower button
-			gtk_widget_set_sensitive( layer_buttons[5], FALSE );	// Delete button
-			gtk_widget_set_sensitive( layer_buttons[6], FALSE );	// Centre button
+			gtk_widget_set_sensitive(layer_tools[LTB_RAISE], layers_total > 0);
+			gtk_widget_set_sensitive(layer_tools[LTB_LOWER], FALSE);
+			gtk_widget_set_sensitive(layer_tools[LTB_DEL], FALSE);
+			gtk_widget_set_sensitive(layer_tools[LTB_CENTER], FALSE);
 			gtk_widget_set_sensitive( layer_trans_toggle, FALSE );
 			gtk_widget_set_sensitive( layer_spin, FALSE );
 			gtk_label_set_text( GTK_LABEL(layer_label_position), _("Background") );
@@ -1172,19 +992,14 @@ static gint layer_select( GtkList *list, GtkWidget *widget, gpointer user_data )
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON( layer_trans_toggle ),
 				layer_table[layer_selected].use_trans);
 
-			gtk_widget_set_sensitive( layer_buttons[1], TRUE );	// Raise button
-			gtk_widget_set_sensitive( layer_buttons[2], TRUE );	// Lower button
-			gtk_widget_set_sensitive( layer_buttons[5], TRUE );	// Delete button
-			gtk_widget_set_sensitive( layer_buttons[6], TRUE );	// Centre button
+			gtk_widget_set_sensitive(layer_tools[LTB_RAISE], j != layers_total);
+			gtk_widget_set_sensitive(layer_tools[LTB_LOWER], TRUE);
+			gtk_widget_set_sensitive(layer_tools[LTB_DEL], TRUE);
+			gtk_widget_set_sensitive(layer_tools[LTB_CENTER], TRUE);
 			gtk_widget_set_sensitive( layer_trans_toggle, TRUE );
 
 			gtk_widget_set_sensitive( layer_spin, TRUE );
 			gtk_spin_button_set_value( GTK_SPIN_BUTTON(layer_spin), layer_table[j].trans);
-
-			if ( j==layers_total )		// Top layer
-			{
-				gtk_widget_set_sensitive( layer_buttons[1], FALSE );	// Raise button
-			}
 		}
 		
 	}
@@ -1197,21 +1012,13 @@ static gint layer_select( GtkList *list, GtkWidget *widget, gpointer user_data )
 
 gint delete_layers_window()
 {
-	int x, y, width, height;
-
 	if ( !GTK_WIDGET_SENSITIVE(layers_window) ) return TRUE;
 		// Stop user prematurely exiting while drag 'n' drop loading
 
-	gdk_window_get_size( layers_window->window, &width, &height );
-	gdk_window_get_root_origin( layers_window->window, &x, &y );
-	
-	inifile_set_gint32("layers_x", x );
-	inifile_set_gint32("layers_y", y );
-	inifile_set_gint32("layers_w", width );
-	inifile_set_gint32("layers_h", height );
+	win_store_pos(layers_window, "layers");
 
 	gtk_widget_destroy(layers_window);
-	men_item_state(menu_layer, TRUE);
+	gtk_widget_set_sensitive(menu_widgets[MENU_LAYER], TRUE);
 	layers_window = NULL;
 
 	return FALSE;
@@ -1241,19 +1048,11 @@ void pressed_paste_layer( GtkMenuItem *menu_item, gpointer user_data )
 			layer_table[layer_selected].x = mem_clip_x;
 			layer_table[layer_selected].y = mem_clip_y;
 
-			mem_pal_copy(layer_table[layer_selected].image->mem_pal,
-				layer_table[ol].image->mem_pal);		// Copy palette
+			mem_pal_copy(layer_table[layer_selected].image->image_.pal,
+				layer_table[ol].image->image_.pal);	// Copy palette
 
-			layer_table[layer_selected].image->tool_pat =
-				layer_table[ol].image->tool_pat;
-			layer_table[layer_selected].image->mem_col_A =
-				layer_table[ol].image->mem_col_A;
-			layer_table[layer_selected].image->mem_col_B =
-				layer_table[ol].image->mem_col_B;
-			layer_table[layer_selected].image->mem_col_A24 =
-				layer_table[ol].image->mem_col_A24;
-			layer_table[layer_selected].image->mem_col_B24 =
-				layer_table[ol].image->mem_col_B24;
+			layer_table[layer_selected].image->state_ =
+				layer_table[ol].image->state_;
 
 			layer_copy_to_main( layer_selected );
 			update_main_with_new_layer();
@@ -1325,8 +1124,8 @@ void move_layer_relative(int l, int change_x, int change_y)	// Move a layer & up
 	}
 	else
 	{
-		lw = layer_table[l].image->mem_width;
-		lh = layer_table[l].image->mem_height;
+		lw = layer_table[l].image->image_.width;
+		lh = layer_table[l].image->image_.height;
 	}
 	layers_notify_changed();
 
@@ -1345,29 +1144,25 @@ void pressed_layers( GtkMenuItem *menu_item, gpointer user_data )
 {
 	GtkWidget *vbox, *hbox, *table, *label, *tog, *scrolledwindow, *item;
 	GtkAccelGroup* ag = gtk_accel_group_new();
-	char txt[256];
+	char txt[32];
 	int i;
 
-	men_item_state(menu_layer, FALSE);
+	gtk_widget_set_sensitive(menu_widgets[MENU_LAYER], FALSE);
 
 	entry_layer_name = NULL;
 	layers_initialized = FALSE;
 
 
 	layers_window = add_a_window( GTK_WINDOW_TOPLEVEL, "", GTK_WIN_POS_NONE, FALSE );
-	gtk_window_set_default_size( GTK_WINDOW(layers_window),
-		inifile_get_gint32("layers_w", 400 ), inifile_get_gint32("layers_h", 400 ) );
-	gtk_widget_set_uposition( layers_window,
-		inifile_get_gint32("layers_x", 0 ), inifile_get_gint32("layers_y", 0 ) );
+	win_restore_pos(layers_window, "layers", 0, 0, 400, 400);
 
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox);
 	gtk_container_add (GTK_CONTAINER (layers_window), vbox);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
 
-	scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
+	scrolledwindow = xpack(vbox, gtk_scrolled_window_new(NULL, NULL));
 	gtk_widget_show (scrolledwindow);
-	gtk_box_pack_start (GTK_BOX (vbox), scrolledwindow, TRUE, TRUE, 0);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow),
 		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
@@ -1388,19 +1183,16 @@ void pressed_layers( GtkMenuItem *menu_item, gpointer user_data )
 		gtk_container_add(GTK_CONTAINER(item), hbox);
 
 		sprintf(txt, "%i", i);
-		label = gtk_label_new( txt );
+		label = pack(hbox, gtk_label_new(txt));
 		gtk_widget_set_usize (label, 40, -2);
 		gtk_misc_set_alignment( GTK_MISC(label), 0.5, 0.5 );
-		gtk_box_pack_start( GTK_BOX(hbox), label, FALSE, FALSE, 0 );
 
-		label = gtk_label_new( "" );
+		label = xpack(hbox, gtk_label_new(""));
 		gtk_misc_set_alignment( GTK_MISC(label), 0, 0.5 );
-		gtk_box_pack_start( GTK_BOX(hbox), label, TRUE, TRUE, 0 );
 		layer_list_data[i].name = label;
 
-		tog = gtk_check_button_new_with_label("");
+		tog = pack(hbox, gtk_check_button_new_with_label(""));
 		gtk_object_set_user_data(GTK_OBJECT(tog), (gpointer)i);
-		gtk_box_pack_start (GTK_BOX(hbox), tog, FALSE, FALSE, 0);
 		layer_list_data[i].toggle = tog;
 		gtk_widget_show_all(item);
 		if ( i == 0 ) gtk_widget_hide(tog);
@@ -1424,12 +1216,12 @@ void pressed_layers( GtkMenuItem *menu_item, gpointer user_data )
 		}
 	}
 
-	layer_iconbar(layers_window, vbox, layer_buttons);
+	pack(vbox, layer_toolbar(layer_tools));
 
 	if ( layers_total == MAX_LAYERS )	// Hide new/duplicate if we have max layers
 	{
-		gtk_widget_set_sensitive( layer_buttons[3], FALSE );
-		gtk_widget_set_sensitive( layer_buttons[4], FALSE );
+		gtk_widget_set_sensitive(layer_tools[LTB_NEW], FALSE);
+		gtk_widget_set_sensitive(layer_tools[LTB_DUP], FALSE );
 	}
 
 	table = add_a_table( 3, 2, 5, vbox );
@@ -1440,7 +1232,7 @@ void pressed_layers( GtkMenuItem *menu_item, gpointer user_data )
 	add_to_table( _("Position"), table, 1, 0, 0 );
 	add_to_table( _("Opacity"), table, 2, 0, 0 );
 
-	entry_layer_name = gtk_entry_new_with_max_length (32);
+	entry_layer_name = gtk_entry_new_with_max_length(LAYER_NAMELEN - 1);
 	gtk_widget_set_usize(entry_layer_name, 100, -2);
 	gtk_widget_show (entry_layer_name);
 	gtk_table_attach (GTK_TABLE (table), entry_layer_name, 1, 2, 0, 1,
@@ -1457,30 +1249,22 @@ void pressed_layers( GtkMenuItem *menu_item, gpointer user_data )
 	mt_spinslide_connect(layer_slider, GTK_SIGNAL_FUNC(layer_inputs_changed), NULL);
 	mt_spinslide_set_value(layer_slider, layer_table[layer_selected].opacity);
 
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
+	hbox = pack(vbox, gtk_hbox_new(FALSE, 0));
+	gtk_widget_show(hbox);
 
 	layer_trans_toggle = add_a_toggle( _("Transparent Colour"), hbox, TRUE );
 	gtk_signal_connect(GTK_OBJECT(layer_trans_toggle), "clicked",
 			GTK_SIGNAL_FUNC(layer_inputs_changed), NULL);
 
-	layer_spin = add_a_spin(0, 0, 255);
-#if GTK_MAJOR_VERSION == 2
-	gtk_signal_connect( GTK_OBJECT( &GTK_SPIN_BUTTON(layer_spin)->entry ),
-			"value_changed", GTK_SIGNAL_FUNC(layer_inputs_changed), NULL);
-#endif
-#if GTK_MAJOR_VERSION == 1
-	gtk_signal_connect( GTK_OBJECT( &GTK_SPIN_BUTTON(layer_spin)->entry ),
-			"changed", GTK_SIGNAL_FUNC(layer_inputs_changed), NULL);
-#endif
-	gtk_box_pack_start (GTK_BOX (hbox), layer_spin, FALSE, FALSE, 0);
+	layer_spin = pack(hbox, add_a_spin(0, 0, 255));
+	spin_connect(layer_spin, GTK_SIGNAL_FUNC(layer_inputs_changed), NULL);
 
 	layer_show_toggle = add_a_toggle( _("Show all layers in main window"), vbox, show_layers_main );
 	gtk_signal_connect(GTK_OBJECT(layer_show_toggle), "clicked",
 			GTK_SIGNAL_FUNC(layer_main_toggled), NULL);
 
-	gtk_widget_add_accelerator (layer_buttons[8], "clicked", ag, GDK_Escape, 0, (GtkAccelFlags) 0);
+	gtk_widget_add_accelerator(layer_tools[LTB_CLOSE], "clicked", ag,
+		GDK_Escape, 0, (GtkAccelFlags) 0);
 
 	gtk_signal_connect_object (GTK_OBJECT (layers_window), "delete_event",
 		GTK_SIGNAL_FUNC (delete_layers_window), NULL);
