@@ -1,11 +1,11 @@
 /*	memory.h
-	Copyright (C) 2004-2006 Mark Tyler and Dmitry Groshev
+	Copyright (C) 2004-2008 Mark Tyler and Dmitry Groshev
 
 	This file is part of mtPaint.
 
 	mtPaint is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 2 of the License, or
+	the Free Software Foundation; either version 3 of the License, or
 	(at your option) any later version.
 
 	mtPaint is distributed in the hope that it will be useful,
@@ -25,6 +25,8 @@
 #define MAX_HEIGHT 16384
 #define MIN_WIDTH 1
 #define MIN_HEIGHT 1
+/* !!! If MAX_WIDTH * MAX_HEIGHT * max bpp won't fit into int, lots of code
+ * !!! will have to be modified to use size_t instead */
 
 #ifdef U_GUADALINEX
 	#define DEFAULT_WIDTH 800
@@ -34,8 +36,26 @@
 	#define DEFAULT_HEIGHT 480
 #endif
 
-#define PALETTE_WIDTH 74
-#define PALETTE_HEIGHT 4500
+/* Palette area layout */
+
+#define PALETTE_SWATCH_X 25
+#define PALETTE_SWATCH_Y 1
+#define PALETTE_SWATCH_W 26
+#define PALETTE_SWATCH_H 16
+#define PALETTE_INDEX_X  0
+#define PALETTE_INDEX_DY 5
+#define PALETTE_DIGIT_W  7
+#define PALETTE_DIGIT_H  7
+#define PALETTE_CROSS_X  53
+#define PALETTE_CROSS_DX 4
+#define PALETTE_CROSS_DY 4
+#define PALETTE_CROSS_W  8
+#define PALETTE_CROSS_H  8
+
+#define PALETTE_WIDTH 70
+#define PALETTE_W3 (PALETTE_WIDTH * 3)
+#define PALETTE_HEIGHT (PALETTE_SWATCH_H * 256 + PALETTE_SWATCH_Y * 2)
+
 #define PATCH_WIDTH 324
 #define PATCH_HEIGHT 324
 
@@ -74,17 +94,57 @@
 #define CMASK_ALL   ((1 << NUM_CHANNELS) - 1)
 #define CMASK_CURR  (1 << mem_channel)
 #define CMASK_FOR(A) (1 << (A))
+#define CMASK_CLIP  ((1 << CHN_IMAGE) | (1 << CHN_ALPHA) | (1 << CHN_SEL))
+
+#define SIZEOF_PALETTE (256 * sizeof(png_color))
 
-char *channames[NUM_CHANNELS + 1], *allchannames[NUM_CHANNELS + 1];
 typedef unsigned char *chanlist[NUM_CHANNELS];
 
-typedef struct
-{
+#define UNDO_TILEMAP_SIZE 32
+
+typedef struct {
 	chanlist img;
-	png_color pal[256];
-	int cols, width, height, bpp;
+	png_color *pal_;
+	unsigned char tilemap[UNDO_TILEMAP_SIZE], *tileptr;
+	int cols, width, height, bpp, flags;
+	size_t size;
 } undo_item;
 
+typedef struct {
+	undo_item *items;	// Pointer to undo images + current image being edited
+	int pointer;		// Index of currently used image on canvas/screen
+	int done;		// Undo images that we have behind current image (i.e. possible UNDO)
+	int redo;		// Undo images that we have ahead of current image (i.e. possible REDO)
+	int max;		// Total number of undo slots
+} undo_stack;
+
+typedef struct {
+	chanlist img;		// Array of pointers to image channels
+	png_color pal[256];	// RGB entries for all 256 palette colours
+	int cols;	// Number of colours in the palette: 1..256 or 0 for no image
+	int bpp;		// Bytes per pixel = 1 or 3
+	int width, height;	// Image geometry
+	size_t size;
+	undo_stack undo_;	// Image's undo stack
+} image_info;
+
+typedef struct {
+	char filename[PATHBUF];		// File name of file loaded/saved
+	char *tempname;			// File name of up-to-date temp file
+	int channel;			// Current active channel
+	int changed;			// Changed since last load/save flag
+	int ics;			// Has the centre been set by the user?
+	float icx, icy;			// Current centre x,y
+	int tool_pat;			// Tool pattern number
+	int xpm_trans;			// Transparent colour index (-1 if none)
+	int xbm_hot_x, xbm_hot_y;	// Current XBM hot spot
+	char prot_mask[256];		// 256 bytes used for indexed images
+	int prot;			// Number of protected colours in prot_RGB
+	int prot_RGB[256];		// Up to 256 RGB colours protected
+	int col_[2];			// Index for colour A & B
+	png_color col_24[2];		// RGB for colour A & B
+} image_state;
+
 /// GRADIENTS
 
 #define MAX_GRAD 65536
@@ -149,9 +209,27 @@ const unsigned char bayer[16];
 int tint_mode[3];			// [0] = off/on, [1] = add/subtract, [2] = button (none, left, middle, right : 0-3)
 
 int mem_cselect;
+int mem_blend;
 int mem_unmask;
 int mem_gradient;
 
+/// BLEND MODE
+
+int blend_mode;
+
+/* Blend modes */
+#define BLEND_NORMAL 0
+#define BLEND_HUE    1
+#define BLEND_SAT    2
+#define BLEND_VALUE  3
+#define BLEND_COLOR  4
+#define BLEND_SATPP  5
+
+#define BLEND_NMODES 6
+#define BLEND_MMASK    0x7F
+#define BLEND_REVERSE  0x80
+#define BLEND_RGBSHIFT 8
+
 /// FLOOD FILL SETTINGS
 
 double flood_step;
@@ -161,36 +239,78 @@ int smudge_mode;
 
 /// IMAGE
 
-char mem_filename[256];			// File name of file loaded/saved
-chanlist mem_img;			// Array of pointers to image channels
-int mem_channel;			// Current active channel
-int mem_img_bpp;			// Bytes per pixel = 1 or 3
-int mem_changed;			// Changed since last load/save flag 0=no, 1=changed
-int mem_width, mem_height;		// Current image geometry
-float mem_icx, mem_icy;			// Current centre x,y
-int mem_ics;				// Has the centre been set by the user? 0=no 1=yes
+#define MIN_UNDO 11	// Number of undo levels + 1
+#define DEF_UNDO 101
+#define MAX_UNDO 1001
 
-unsigned char *mem_clipboard;		// Pointer to clipboard data
-unsigned char *mem_clip_mask;		// Pointer to clipboard mask
-unsigned char *mem_clip_alpha;		// Pointer to clipboard alpha
-unsigned char *mem_brushes;		// Preset brushes image
+int mem_undo_depth;				// Current undo depth
+
+image_info mem_image;			// Current image
+
+#define mem_img		mem_image.img		
+#define mem_pal		mem_image.pal
+#define mem_cols	mem_image.cols
+#define mem_img_bpp	mem_image.bpp
+#define mem_width	mem_image.width
+#define mem_height	mem_image.height
+
+#define mem_undo_im_		mem_image.undo_.items
+#define mem_undo_pointer	mem_image.undo_.pointer
+#define mem_undo_done		mem_image.undo_.done
+#define mem_undo_redo		mem_image.undo_.redo
+#define mem_undo_max		mem_image.undo_.max
+
+image_info mem_clip;			// Current clipboard
+
+#define mem_clipboard		mem_clip.img[CHN_IMAGE]
+#define mem_clip_alpha		mem_clip.img[CHN_ALPHA]
+#define mem_clip_mask		mem_clip.img[CHN_SEL]
+#define mem_clip_bpp		mem_clip.bpp
+#define mem_clip_w		mem_clip.width
+#define mem_clip_h		mem_clip.height
+
+// Always use undo slot #1 for clipboard backup
+#define OLD_CLIP 1
+// mem_clip.undo_.done == 0 means no backup clipboard
+#define HAVE_OLD_CLIP		(mem_clip.undo_.done)
+#define mem_clip_real_img	mem_clip.undo_.items[OLD_CLIP].img
+#define mem_clip_real_w		mem_clip.undo_.items[OLD_CLIP].width
+#define mem_clip_real_h		mem_clip.undo_.items[OLD_CLIP].height
+#define mem_clip_real_clear()	mem_free_image(&mem_clip, FREE_UNDO)
+
+image_state mem_state;			// Current edit settings
+
+#define mem_filename		mem_state.filename
+#define mem_tempname		mem_state.tempname
+#define mem_channel		mem_state.channel
+#define mem_changed		mem_state.changed
+#define mem_icx			mem_state.icx
+#define mem_icy			mem_state.icy
+#define mem_ics			mem_state.ics
+#define mem_tool_pat		mem_state.tool_pat
+#define mem_xpm_trans		mem_state.xpm_trans
+#define mem_xbm_hot_x		mem_state.xbm_hot_x
+#define mem_xbm_hot_y		mem_state.xbm_hot_y
+#define mem_prot_mask		mem_state.prot_mask
+#define mem_prot_RGB		mem_state.prot_RGB
+#define mem_prot		mem_state.prot
+#define mem_col_		mem_state.col_
+#define mem_col_A		mem_state.col_[0]
+#define mem_col_B		mem_state.col_[1]
+#define mem_col_24		mem_state.col_24
+#define mem_col_A24		mem_state.col_24[0]
+#define mem_col_B24		mem_state.col_24[1]
+
+int mem_clip_x, mem_clip_y;		// Clipboard location on canvas
+
+extern unsigned char mem_brushes[];	// Preset brushes image
 int brush_tool_type;			// Last brush tool type
 int mem_brush_list[81][3];		// Preset brushes parameters
-int mem_clip_bpp;			// Bytes per pixel
-int mem_clip_w, mem_clip_h;		// Clipboard geometry
-int mem_clip_x, mem_clip_y;		// Clipboard location on canvas
 int mem_nudge;				// Nudge pixels per SHIFT+Arrow key during selection/paste
 
 int mem_preview;			// Preview an RGB change
 int mem_prev_bcsp[6];			// BR, CO, SA, POSTERIZE, GAMMA, Hue
 
-#define MAX_UNDO 101			// Maximum number of undo levels + 1
-
-undo_item mem_undo_im_[MAX_UNDO];	// Pointers to undo images + current image being edited
-
-int mem_undo_pointer;		// Pointer to currently used image on canvas/screen
-int mem_undo_done;		// Undo images that we have behind current image (i.e. possible UNDO)
-int mem_undo_redo;		// Undo images that we have ahead of current image (i.e. possible REDO)
 int mem_undo_limit;		// Max MB memory allocation limit
 int mem_undo_opacity;		// Use previous image for opacity calculations?
 
@@ -201,45 +321,26 @@ unsigned char mem_grid_rgb[3];		// RGB colour of grid
 
 /// PATTERNS
 
-char mem_patterns[81][8][8];		// Pattern bitmaps
-unsigned char *mem_pattern;		// Original 0-1 pattern
-unsigned char *mem_col_pat;		// Indexed 8x8 colourised pattern using colours A & B
-unsigned char *mem_col_pat24;		// RGB 8x8 colourised pattern using colours A & B
+unsigned char mem_pattern[8 * 8];	// Current pattern
+unsigned char mem_col_pat[8 * 8];	// Indexed 8x8 colourised pattern using colours A & B
+unsigned char mem_col_pat24[8 * 8 * 3];	// RGB 8x8 colourised pattern using colours A & B
 
 /// PREVIEW/TOOLS
 
 int tool_type, tool_size, tool_flow;	// Currently selected tool
 int tool_opacity;			// Opacity - 255 = solid
-int tool_pat;				// Tool pattern number
 int pen_down;				// Are we drawing? - Used to see if we need to do an UNDO
 int tool_ox, tool_oy;			// Previous tool coords - used by continuous mode
 int mem_continuous;			// Area we painting the static shapes continuously?
 
 int mem_brcosa_allow[3];		// BRCOSA RGB
 
-/// FILE
-
-int mem_xpm_trans;			// Current XPM file transparency colour index
-int mem_xbm_hot_x, mem_xbm_hot_y;	// Current XBM hot spot
-int mem_jpeg_quality;			// JPEG quality setting
-
 /// PALETTE
 
-png_color mem_pal[256];			// RGB entries for all 256 palette colours
 png_color mem_pal_def[256];		// Default palette entries for new image
 int mem_pal_def_i;			// Items in default palette
-int mem_cols;				// Number of colours in the palette: 2..256 or 0 for no image
-char *mem_pals;				// RGB screen memory holding current palette
-char mem_prot_mask[256];		// 256 bytes used for indexed images
-int mem_prot_RGB[256];			// Up to 256 RGB colours protected
-int mem_prot;				// 0..256 : Number of protected colours in mem_prot_RGB
+extern unsigned char mem_pals[];	// RGB screen memory holding current palette
 
-int mem_col_[2];			// Index for colour A & B
-#define mem_col_A mem_col_[0]
-#define mem_col_B mem_col_[1]
-png_color mem_col_24[2];		// RGB for colour A & B
-#define mem_col_A24 mem_col_24[0]
-#define mem_col_B24 mem_col_24[1]
 int mem_background;			// Non paintable area
 int mem_histogram[256];
 
@@ -254,7 +355,12 @@ void line_nudge(linedata line, int x, int y);
 
 /// Procedures
 
-void init_istate();	// Set initial state of image variables
+void init_istate();		// Set initial state of image variables
+int init_undo(undo_stack *ustack, int depth);	// Create new undo stack of a given depth
+void update_undo_depth();	// Resize all undo stacks
+
+void mem_free_chanlist(chanlist img);
+int cmask_from(chanlist img);	// Chanlist to cmask
 
 int mem_count_all_cols();			// Count all colours - Using main image
 int mem_count_all_cols_real(unsigned char *im, int w, int h);	// Count all colours - very memory greedy
@@ -265,43 +371,67 @@ int mem_cols_used_real(unsigned char *im, int w, int h, int max_count, int prog)
 void mem_cols_found_dl(unsigned char userpal[3][256]);		// Convert results ready for DL code
 
 
-int get_next_line(char *input, int length, FILE *fp);		// Get next length chars of text file
-int mt_round( float n );			// Round a float to nearest whole number
 int read_hex( char in );			// Convert character to hex value 0..15.  -1=error
 int read_hex_dub( char *in );			// Read hex double
 
-char *grab_memory( int size, char byte );	// Malloc memory, reset all bytes
-void mem_clear();				// Remove old image if any
+#define FREE_IMAGE 1
+#define FREE_UNDO  2
+#define FREE_ALL   3
+
+//	Clear/remove image data
+void mem_free_image(image_info *image, int mode);
+//	Allocate new image data
+int mem_alloc_image(image_info *image, int w, int h, int bpp, int cmask,
+	chanlist src);
 //	Allocate space for new image, removing old if needed
 int mem_new( int width, int height, int bpp, int cmask );
-int valid_file( char *filename );		// Can this file be opened for reading?
+//	Allocate new clipboard, removing or preserving old as needed
+int mem_clip_new(int width, int height, int bpp, int cmask, int backup);
+
+int load_def_palette(char *name);
+int load_def_patterns(char *name);
 void mem_init();				// Initialise memory
 
-int mem_used();				// Return the number of bytes used in image + undo stuff
-int mem_used_layers();			// Return the number of bytes used in image + undo in all layers
+//	Return the number of bytes used in image + undo stuff
+size_t mem_used();
+//	Return the number of bytes used in image + undo in all layers
+size_t mem_used_layers();
 
 void mem_bacteria( int val );			// Apply bacteria effect val times the canvas area
 void do_effect( int type, int param );		// 0=edge detect 1=UNUSED 2=emboss
 void mem_gauss(double radiusX, double radiusY, int gcor);
 void mem_unsharp(double radius, double amount, int threshold, int gcor);
+void mem_dog(double radiusW, double radiusN, int norm, int gcor);
+
 
 /// PALETTE PROCS
 
 int mem_load_pal( char *file_name, png_color *pal );	// Load file into palette array >1 => cols read
 void mem_pal_load_def();		// Load default palette
 
-#define mem_pal_copy(A, B) memcpy((A), (B), sizeof(png_color) * 256)
+#define mem_pal_copy(A, B) memcpy((A), (B), SIZEOF_PALETTE)
 void mem_pal_init();			// Initialise whole of palette RGB
 int mem_pal_cmp( png_color *pal1,	// Count itentical palette entries
 	png_color *pal2 );
 void mem_greyscale(int gcor);		// Convert image to greyscale
 int mem_convert_rgb();			// Convert image to RGB
-int mem_convert_indexed();		// Convert image to Indexed Palette
+int mem_convert_indexed(int img);	// Convert image to Indexed Palette
+//	Quantize image using Max-Min algorithm
+int maxminquan(unsigned char *inbuf, int width, int height, int quant_to,
+	unsigned char userpal[3][256]);
+//	Quantize image using PNN algorithm
+int pnnquan(unsigned char *inbuf, int width, int height, int quant_to,
+	unsigned char userpal[3][256]);
 //	Convert RGB->indexed using error diffusion with variety of options
-int mem_dither(unsigned char *old, int ncols, short *dither, int cspace, int dist,
-	int limit, int selc, int serpent, double emult);
+int mem_dither(unsigned char *old, int ncols, short *dither, int cspace,
+	int dist, int limit, int selc, int serpent, int rgb8b, double emult);
+//	Do the same in dumb but fast way
+int mem_dumb_dither(unsigned char *old, unsigned char *new, png_color *pal,
+	int width, int height, int ncols, int dither);
+//	Set up colors A, B, and pattern for dithering a given RGB color
+void mem_find_dither(int red, int green, int blue); 
+//	Convert image to Indexed Palette using quantize
 int mem_quantize( unsigned char *old_mem_image, int target_cols, int type );
-					// Convert image to Indexed Palette using quantize
 void mem_invert();			// Invert the palette
 
 void mem_mask_setall(char val);		// Clear/set all masks
@@ -316,8 +446,8 @@ int scan_duplicates();			// Find duplicate palette colours
 void remove_duplicates();		// Remove duplicate palette colours - call AFTER scan_duplicates
 int mem_remove_unused_check();		// Check to see if we can remove unused palette colours
 int mem_remove_unused();		// Remove unused palette colours
-void mem_scale_pal( int i1, int r1, int g1, int b1, int i2, int r2, int g2, int b2 );
-					// Generate a scaled palette
+void mem_scale_pal(png_color *pal, int i1, int r1, int g1, int b1,
+	int i2, int r2, int g2, int b2); // Generate a scaled palette
 //	Create colour-transformed palette
 void transform_pal(png_color *pal1, png_color *pal2, int p1, int p2);
 void mem_pal_sort( int a, int i1, int i2, int rev );
@@ -343,21 +473,24 @@ void rgb2hsv(unsigned char *rgb, double *hsv);
 #define UNDO_PASTE 7	/* Paste operation (current / RGBA) */
 #define UNDO_TOOL  8	/* Same as UNDO_DRAW but respects pen_down */
 
-
-void undo_free_x(undo_item *undo);
 int mem_undo_next(int mode);		// Call this after a draw event but before any changes to image
 //	 Get address of previous channel data (or current if none)
 unsigned char *mem_undo_previous(int channel);
+void mem_undo_prepare();	// Call this after changes to image, to compress last frame
 
 void mem_undo_backward();		// UNDO requested by user
 void mem_undo_forward();		// REDO requested by user
 
-#define UC_CREATE  1	/* Force create */
-#define UC_NOCOPY  2	/* Forbid copy */
-#define UC_DELETE  4	/* Force delete */
-#define UC_PENDOWN 8	/* Respect pen_down */
+#define UC_CREATE  0x01	/* Force create */
+#define UC_NOCOPY  0x02	/* Forbid copy */
+#define UC_DELETE  0x04	/* Force delete */
+#define UC_PENDOWN 0x08	/* Respect pen_down */
+#define UC_GETMEM  0x10 /* Get memory and do nothing */
 
 int undo_next_core(int mode, int new_width, int new_height, int new_bpp, int cmask);
+void update_undo(image_info *image);	// Copy image state into current undo frame
+//	Try to allocate a memory block, releasing undo frames if needed
+void *mem_try_malloc(size_t size);
 
 
 //// Drawing Primitives
@@ -385,13 +518,25 @@ void mem_flip_v(char *mem, char *tmp, int w, int h, int bpp);	// Flip image vert
 void mem_flip_h( char *mem, int w, int h, int bpp );		// Flip image horizontally
 int mem_sel_rot( int dir );					// Rotate clipboard 90 degrees
 int mem_image_rot( int dir );					// Rotate canvas 90 degrees
-int mem_rotate_free(double angle, int type, int gcor);		// Rotate canvas by any angle (degrees)
-int mem_image_scale(int nw, int nh, int type, int gcor);	// Scale image
+
+void mem_rotate_geometry(int ow, int oh, double angle, int *nw, int *nh);
+				// Get new image geometry of rotation. angle = degrees
+// Rotate canvas or clipboard by any angle (degrees)
+int mem_rotate_free(double angle, int type, int gcor, int clipboard);
+void mem_rotate_free_real(chanlist old_img, chanlist new_img, int ow, int oh,
+	int nw, int nh, int bpp, double angle, int mode, int gcor, int dis_a,
+	int silent);
+
+int mem_image_scale(int nw, int nh, int type, int gcor, int sharp);	// Scale image
+int mem_image_scale_real(chanlist old_img, int ow, int oh, int bpp, chanlist new_img, int nw, int nh, int type, int gcor, int sharp);
 int mem_image_resize(int nw, int nh, int ox, int oy, int mode);	// Resize image
 
 int mem_isometrics(int type);
 
-void mem_threshold(int channel, int level);			// Threshold channel values
+void mem_threshold(unsigned char *img, int len, int level);	// Threshold channel values
+void mem_demultiply(unsigned char *img, unsigned char *alpha, int len, int bpp);
+
+void set_xlate(unsigned char *xlat, int bpp);			// Build bitdepth translation table
 
 void flood_fill( int x, int y, unsigned int target );
 
@@ -400,14 +545,13 @@ void tline( int x1, int y1, int x2, int y2, int size );		// Draw size thickness
 void g_para( int x1, int y1, int x2, int y2, int xv, int yv );	// Draw general parallelogram
 void f_rectangle( int x, int y, int w, int h );			// Draw a filled rectangle
 void f_circle( int x, int y, int r );				// Draw a filled circle
-void f_ellipse( int x1, int y1, int x2, int y2 );		// Draw a filled ellipse
-void o_ellipse( int x1, int y1, int x2, int y2, int thick );	// Draw an ellipse outline
+void mem_ellipse( int x1, int y1, int x2, int y2, int thick );	// Thickness 0 means filled
 
 // Draw whatever is bounded by two pairs of lines
 void draw_quad(linedata line1, linedata line2, linedata line3, linedata line4);
 
 //	A couple of shorthands to get an int representation of an RGB colour
-#define PNG_2_INT(var) ((var.red << 16) + (var.green << 8) + (var.blue))
+#define PNG_2_INT(P) (((P).red << 16) + ((P).green << 8) + ((P).blue))
 #define MEM_2_INT(M,I) (((M)[(I)] << 16) + ((M)[(I) + 1] << 8) + (M)[(I) + 2])
 #define INT_2_R(A) ((A) >> 16)
 #define INT_2_G(A) (((A) >> 8) & 0xFF)
