@@ -1,11 +1,11 @@
 /*	prefs.c
-	Copyright (C) 2005-2007 Mark Tyler and Dmitry Groshev
+	Copyright (C) 2005-2008 Mark Tyler and Dmitry Groshev
 
 	This file is part of mtPaint.
 
 	mtPaint is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 2 of the License, or
+	the Free Software Foundation; either version 3 of the License, or
 	(at your option) any later version.
 
 	mtPaint is distributed in the hope that it will be useful,
@@ -17,10 +17,6 @@
 	along with mtPaint in the file COPYING.
 */
 
-#include <gtk/gtk.h>
-#include <gdk/gdkkeysyms.h>
-#include <math.h>
-
 #include "global.h"
 #include "mygtk.h"
 #include "memory.h"
@@ -36,39 +32,47 @@
 ///	PREFERENCES WINDOW
 
 GtkWidget *prefs_window, *prefs_status[STATUS_ITEMS];
-static GtkWidget *spinbutton_maxmem, *spinbutton_greys, *spinbutton_nudge, *spinbutton_pan;
-static GtkWidget *spinbutton_trans, *spinbutton_hotx, *spinbutton_hoty, *spinbutton_jpeg,
-	*spinbutton_recent, *spinbutton_silence;
+static GtkWidget *spinbutton_maxmem, *spinbutton_maxundo, *spinbutton_greys,
+	*spinbutton_nudge, *spinbutton_pan;
+static GtkWidget *spinbutton_trans, *spinbutton_hotx, *spinbutton_hoty,
+	*spinbutton_jpeg, *spinbutton_jp2, *spinbutton_png, *spinbutton_recent,
+	*spinbutton_silence;
+static GtkWidget *checkbutton_tgaRLE, *checkbutton_tga565, *checkbutton_tgadef,
+	*checkbutton_undo;
+
 static GtkWidget *checkbutton_paste, *checkbutton_cursor, *checkbutton_exit, *checkbutton_quit;
 static GtkWidget *checkbutton_zoom[4],		// zoom 100%, wheel, optimize cheq, disable trans
 	*checkbutton_commit, *checkbutton_center, *checkbutton_gamma;
-GtkWidget *clipboard_entry, *entry_handbook[2];
+static GtkWidget *clipboard_entry, *entry_handbook[2], *entry_def[2];
 static GtkWidget *spinbutton_grid[4];
 static GtkWidget *check_tablet[3], *hscale_tablet[3], *label_tablet_device, *label_tablet_pressure;
 
 static char	*tablet_ini[] = { "tablet_value_size", "tablet_value_flow", "tablet_value_opacity" },
-		*tablet_ini2[] = { "tablet_use_size", "tablet_use_flow", "tablet_use_opacity" },
-		*tablet_ini3[] = { "tablet_name", "tablet_mode", "tablet_axes_v" };
+		*tablet_ini2[] = { "tablet_use_size", "tablet_use_flow", "tablet_use_opacity" };
 
 #if GTK_MAJOR_VERSION == 1
-static GdkDeviceInfo *tablet_device = NULL;
+static GdkDeviceInfo *tablet_device;
 #endif
 #if GTK_MAJOR_VERSION == 2
-static GdkDevice *tablet_device = NULL;
+static GdkDevice *tablet_device;
 #endif
 
-gboolean tablet_working = FALSE;		// Has the device been initialized?
+int tablet_working;		// Has the device been initialized?
 
-gboolean tablet_tool_use[3];			// Size, flow, opacity
+int tablet_tool_use[3];				// Size, flow, opacity
 float tablet_tool_factor[3];			// Size, flow, opacity
 
 
 #ifdef U_NLS
 
-#define PREF_LANGS 12
+#define PREF_LANGS 16
 
-char	*pref_lang_ini_code[PREF_LANGS] = { "system", "zh_TW.utf8", "cs_CZ", "en_GB", "fr_FR", "de_DE", 
-		"pl_PL", "pt_PT", "pt_BR", "sk_SK", "es_ES", "tr_TR" };
+char *pref_lang_ini_code[PREF_LANGS] = { "system",
+	"zh_CN.utf8", "zh_TW.utf8",
+	"cs_CZ", "en_GB", "fr_FR", "gl_ES",
+	"de_DE", "ja_JP.utf8", "pl_PL", "pt_PT",
+	"pt_BR", "ru_RU", "sk_SK",
+	"es_ES", "tr_TR" };
 
 int pref_lang;
 
@@ -93,11 +97,6 @@ static gint expose_tablet_preview( GtkWidget *widget, GdkEventExpose *event )
 	return FALSE;
 }
 
-void click_file_browse( GtkWidget *widget, gpointer data )
-{
-	file_selector( (int) data );
-}
-
 
 static GtkWidget *inputd = NULL;
 
@@ -116,8 +115,8 @@ gint delete_inputd( GtkWidget *widget, GdkEvent *event, gpointer data )
 
 	if ( tablet_working )		// Store tablet settings in INI file for future session
 	{
-		inifile_set( tablet_ini3[0], dev->name );
-		inifile_set_gint32( tablet_ini3[1], dev->mode );
+		inifile_set( "tablet_name", dev->name );
+		inifile_set_gint32( "tablet_mode", dev->mode );
 
 		for ( i=0; i<dev->num_axes; i++ )
 		{
@@ -127,12 +126,11 @@ gint delete_inputd( GtkWidget *widget, GdkEvent *event, gpointer data )
 #if GTK_MAJOR_VERSION == 2
 			j = dev->axes[i].use;
 #endif
-			sprintf(txt, "%s%i", tablet_ini3[2], i);
+			sprintf(txt, "tablet_axes_v%i", i);
 			inifile_set_gint32( txt, j );
 		}
 	}
 
-	inifile_set_gboolean( "tablet_USE", tablet_working );
 	gtk_widget_destroy(inputd);
 	inputd = NULL;
 
@@ -143,7 +141,7 @@ static void delete_prefs(GtkWidget *widget)
 {
 	if ( inputd != NULL ) delete_inputd( NULL, NULL, NULL );
 	gtk_widget_destroy(prefs_window);
-	men_item_state( menu_prefs, TRUE );
+	gtk_widget_set_sensitive(menu_widgets[MENU_PREFS], TRUE);
 	clipboard_entry = NULL;
 }
 
@@ -256,24 +254,26 @@ gint conf_tablet( GtkWidget *widget, GdkEvent *event, gpointer data )
 
 static void prefs_apply(GtkWidget *widget)
 {
+	char path[PATHBUF];
 	int i, j;
-	char txt[64];
 
 	for ( i=0; i<STATUS_ITEMS; i++ )
 	{
-		sprintf(txt, "status%iToggle", i);
-		inifile_set_gboolean( txt,
-			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_status[i])) );
-		status_on[i] = inifile_get_gboolean(txt, TRUE);
+		status_on[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_status[i]));
 	}
 
 	mem_undo_limit = read_spin(spinbutton_maxmem);
+	mem_undo_depth = read_spin(spinbutton_maxundo);
 	mem_background = read_spin(spinbutton_greys);
 	mem_nudge = read_spin(spinbutton_nudge);
+	max_pan = read_spin(spinbutton_pan);
+
 	mem_xpm_trans = read_spin(spinbutton_trans);
 	mem_xbm_hot_x = read_spin(spinbutton_hotx);
 	mem_xbm_hot_y = read_spin(spinbutton_hoty);
-	mem_jpeg_quality = read_spin(spinbutton_jpeg);
+	jpeg_quality = read_spin(spinbutton_jpeg);
+	jp2_rate = read_spin(spinbutton_jp2);
+	png_compression = read_spin(spinbutton_png);
 	recent_files = read_spin(spinbutton_recent);
 	silence_limit = read_spin(spinbutton_silence);
 
@@ -290,24 +290,18 @@ static void prefs_apply(GtkWidget *widget)
 		tablet_tool_factor[i] = j / 100.0;
 	}
 
-	inifile_set_gint32( "gridMin", mem_grid_min );
 	inifile_set_gint32( "gridR", mem_grid_rgb[0] );
 	inifile_set_gint32( "gridG", mem_grid_rgb[1] );
 	inifile_set_gint32( "gridB", mem_grid_rgb[2] );
 
 
-	inifile_set_gint32( "panSize", read_spin(spinbutton_pan));
-	inifile_set_gint32( "undoMBlimit", mem_undo_limit );
-	inifile_set_gint32( "backgroundGrey", mem_background );
-	inifile_set_gint32( "pixelNudge", mem_nudge );
-	inifile_set_gint32( "jpegQuality", mem_jpeg_quality );
-	inifile_set_gint32( "recentFiles", recent_files );
-	inifile_set_gint32( "silence_limit", silence_limit );
+	tga_RLE = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_tgaRLE)) ? 1 : 0;
+	tga_565 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_tga565));
+	tga_defdir = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_tgadef));
+	undo_load = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_undo));
 
-	inifile_set_gboolean( "pasteToggle",
-		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_paste)) );
-	inifile_set_gboolean( "cursorToggle",
-		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_cursor)) );
+	show_paste = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_paste));
+	cursor_tool = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_cursor));
 	inifile_set_gboolean( "exitToggle",
 		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_exit)) );
 
@@ -318,15 +312,12 @@ static void prefs_apply(GtkWidget *widget)
 		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_zoom[1])) );
 #endif
 	chequers_optimize = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_zoom[2]));
-	inifile_set_gboolean( "optimizeChequers", chequers_optimize );
 	opaque_view = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_zoom[3]));
-	inifile_set_gboolean( "disableTransparency", FALSE );
 
 	inifile_set_gboolean( "pasteCommit",
 		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_commit)) );
 
 	q_quit = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_quit));
-	inifile_set_gboolean( "quitToggle", q_quit );
 
 	inifile_set_gboolean("centerSettings",
 		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_center)));
@@ -336,16 +327,29 @@ static void prefs_apply(GtkWidget *widget)
 #ifdef U_NLS
 	inifile_set("languageSETTING", pref_lang_ini_code[pref_lang]);
 	setup_language();
+	string_init();				// Translate static strings
 #endif
 
-	strncpy(mem_clip_file, gtk_entry_get_text(GTK_ENTRY(clipboard_entry)), 250);
+	gtkncpy(mem_clip_file, gtk_entry_get_text(GTK_ENTRY(clipboard_entry)), PATHBUF);
 	inifile_set("clipFilename", mem_clip_file);
 
-	inifile_set(HANDBOOK_BROWSER_INI, (gchar *)gtk_entry_get_text(GTK_ENTRY(entry_handbook[0])) );
-	inifile_set(HANDBOOK_LOCATION_INI, (gchar *)gtk_entry_get_text(GTK_ENTRY(entry_handbook[1])) );
+	gtkncpy(path, gtk_entry_get_text(GTK_ENTRY(entry_handbook[0])), PATHBUF);
+	inifile_set(HANDBOOK_BROWSER_INI, path);
 
-	show_paste = inifile_get_gboolean( "pasteToggle", TRUE );
+	gtkncpy(path, gtk_entry_get_text(GTK_ENTRY(entry_handbook[1])), PATHBUF);
+	inifile_set(HANDBOOK_LOCATION_INI, path);
 
+	gtkncpy(path, gtk_entry_get_text(GTK_ENTRY(entry_def[0])), PATHBUF);
+	if (strcmp(path, inifile_get(DEFAULT_PAL_INI, "")))
+		load_def_palette(path);
+	inifile_set(DEFAULT_PAL_INI, path);
+
+	gtkncpy(path, gtk_entry_get_text(GTK_ENTRY(entry_def[1])), PATHBUF);
+	if (strcmp(path, inifile_get(DEFAULT_PAT_INI, "")))
+		load_def_patterns(path);
+	inifile_set(DEFAULT_PAT_INI, path);
+
+	update_undo_depth();		// If undo depth was changed
 	update_all_views();		// Update canvas for changes
 	set_cursor();
 
@@ -407,57 +411,38 @@ static gint tablet_preview_motion(GtkWidget *widget, GdkEventMotion *event)
 	return TRUE;
 }
 
-static GtkWidget *path_box(char *name, GtkWidget *box, int fsmode)
-{
-	GtkWidget *hbox, *label, *entry, *button;
- 
-	label = gtk_label_new(name);
-	gtk_widget_show(label);
-	gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 2);
-
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_widget_show(hbox);
-	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 2);
-
-	entry = gtk_entry_new();
-	gtk_widget_show(entry);
-	gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 5);
-
-	button = add_a_button(_("Browse"), 2, hbox, FALSE);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		GTK_SIGNAL_FUNC(click_file_browse), (gpointer)fsmode);
-
-	return (entry);
-}
-
 void pressed_preferences( GtkMenuItem *menu_item, gpointer user_data )
 {
 	int i;
 #ifdef U_NLS
-	char *pref_langs[PREF_LANGS] = { _("Default System Language"), _("Chinese (Taiwanese)"),
-		_("Czech"), _("English (UK)"),
-		_("French"), _("German"), _("Polish"), _("Portuguese"), _("Portuguese (Brazilian)"),
-		_("Slovak"), _("Spanish"), _("Turkish")
-					};
+	char *pref_langs[PREF_LANGS] = { _("Default System Language"),
+		_("Chinese (Simplified)"), _("Chinese (Taiwanese)"),
+		_("Czech"), _("English (UK)"), _("French"), _("Galician"),
+		_("German"), _("Japanese"), _("Polish"), _("Portuguese"),
+		_("Portuguese (Brazilian)"), _("Russian"), _("Slovak"),
+		_("Spanish"), _("Turkish") };
 #endif
 
 
 	GtkWidget *vbox3, *hbox4, *table3, *table4, *table5, *drawingarea_tablet;
-	GtkWidget *button1, *notebook1, *vbox_1, *vbox_2, *vbox_3, *label;
+	GtkWidget *button1, *notebook1, *page, *vbox_2, *label;
 
-	char *tab_tex[] = { _("Max memory used for undo (MB)"), _("Greyscale backdrop"),
+	char *tab_tex[] = { _("Max memory used for undo (MB)"),
+		_("Max undo levels"), _("Greyscale backdrop"),
 		_("Selection nudge pixels"), _("Max Pan Window Size") };
 	char *tab_tex2[] = { _("Transparency index"), _("XBM X hotspot"), _("XBM Y hotspot"),
-		_("JPEG Save Quality (100=High)   "), _("Recently Used Files"),
+		_("JPEG Save Quality (100=High)"), _("JPEG2000 Compression (0=Lossless)"),
+		_("PNG Compression (0=None)"), _("Recently Used Files"),
 		_("Progress bar silence limit") };
 	char *tab_tex3[] = { _("Minimum grid zoom"), _("Grid colour RGB") };
 	char *stat_tex[] = { _("Canvas Geometry"), _("Cursor X,Y"),
 		_("Pixel [I] {RGB}"), _("Selection Geometry"), _("Undo / Redo") },
 		*tablet_txt[] = { _("Size"), _("Flow"), _("Opacity") };
-	char txt[64];
+	char txt[PATHTXT];
 
 
-	men_item_state( menu_prefs, FALSE );	// Make sure the user can only open 1 prefs window
+	// Make sure the user can only open 1 prefs window
+	gtk_widget_set_sensitive(menu_widgets[MENU_PREFS], FALSE);
 
 	prefs_window = add_a_window( GTK_WINDOW_TOPLEVEL, _("Preferences"), GTK_WIN_POS_CENTER, FALSE );
 
@@ -467,116 +452,106 @@ void pressed_preferences( GtkMenuItem *menu_item, gpointer user_data )
 
 ///	SETUP NOTEBOOK
 
-	notebook1 = gtk_notebook_new ();
-	gtk_box_pack_start (GTK_BOX (vbox3), notebook1, TRUE, TRUE, 0);
+	notebook1 = xpack(vbox3, gtk_notebook_new());
 	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook1), GTK_POS_TOP);
 	gtk_widget_show (notebook1);
 
 ///	---- TAB1 - GENERAL
 
-	vbox_1 = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox_1);
-	gtk_container_add (GTK_CONTAINER (notebook1), vbox_1);
-
-	label = gtk_label_new( _("General") );
-	gtk_widget_show (label);
-	gtk_notebook_set_tab_label(GTK_NOTEBOOK (notebook1),
-		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), label);
-
-
-	table3 = add_a_table( 3, 2, 10, vbox_1 );
+	page = add_new_page(notebook1, _("General"));
+	table3 = add_a_table( 5, 2, 10, page );
 
 ///	TABLE TEXT
-	for ( i=0; i<4; i++ ) add_to_table( tab_tex[i], table3, i, 0, 5 );
+	for (i = 0; i < 5; i++ ) add_to_table( tab_tex[i], table3, i, 0, 5 );
 
 ///	TABLE SPINBUTTONS
-	spinbutton_maxmem = spin_to_table(table3, 0, 1, 5, inifile_get_gint32("undoMBlimit", 32 ), 1, 1000);
-	spinbutton_greys = spin_to_table(table3, 1, 1, 5, inifile_get_gint32("backgroundGrey", 180 ), 0, 255);
-	spinbutton_nudge = spin_to_table(table3, 2, 1, 5, inifile_get_gint32("pixelNudge", 8 ), 2, 512);
-	spinbutton_pan = spin_to_table(table3, 3, 1, 5, inifile_get_gint32("panSize", 128 ), 64, 256);
+	spinbutton_maxmem = spin_to_table(table3, 0, 1, 5, mem_undo_limit, 1, 2048);
+	spinbutton_maxundo = spin_to_table(table3, 1, 1, 5, mem_undo_depth & ~1,
+		MIN_UNDO & ~1, MAX_UNDO & ~1);
+	spinbutton_greys = spin_to_table(table3, 2, 1, 5, mem_background, 0, 255);
+	spinbutton_nudge = spin_to_table(table3, 3, 1, 5, mem_nudge, 2, 512);
+	spinbutton_pan = spin_to_table(table3, 4, 1, 5, max_pan, 64, 256);
 
 	checkbutton_paste = add_a_toggle( _("Display clipboard while pasting"),
-		vbox_1, inifile_get_gboolean("pasteToggle", TRUE) );
+		page, show_paste );
 	checkbutton_cursor = add_a_toggle( _("Mouse cursor = Tool"),
-		vbox_1, inifile_get_gboolean("cursorToggle", TRUE) );
+		page, cursor_tool );
 	checkbutton_exit = add_a_toggle( _("Confirm Exit"),
-		vbox_1, inifile_get_gboolean("exitToggle", FALSE) );
+		page, inifile_get_gboolean("exitToggle", FALSE) );
 	checkbutton_quit = add_a_toggle( _("Q key quits mtPaint"),
-		vbox_1, inifile_get_gboolean("quitToggle", TRUE) );
+		page, q_quit );
 	checkbutton_commit = add_a_toggle( _("Changing tool commits paste"),
-		vbox_1, inifile_get_gboolean("pasteCommit", FALSE) );
+		page, inifile_get_gboolean("pasteCommit", FALSE) );
 	checkbutton_center = add_a_toggle(_("Centre tool settings dialogs"),
-		vbox_1, inifile_get_gboolean("centerSettings", TRUE));
+		page, inifile_get_gboolean("centerSettings", TRUE));
 	checkbutton_gamma = add_a_toggle(_("Use gamma correction by default"),
-		vbox_1, inifile_get_gboolean("defaultGamma", FALSE));
+		page, inifile_get_gboolean("defaultGamma", FALSE));
 
 
 ///	---- TAB2 - FILES
 
-	vbox_2 = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox_2);
-	gtk_container_add (GTK_CONTAINER (notebook1), vbox_2);
+	page = add_new_page(notebook1, _("Files"));
+	table4 = add_a_table(7, 2, 10, page);
 
-	label = gtk_label_new( _("Files") );
-	gtk_widget_show (label);
-	gtk_notebook_set_tab_label(GTK_NOTEBOOK (notebook1),
-		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), label);
-
-	table4 = add_a_table( 5, 2, 10, vbox_2 );
-
-	for ( i=0; i<6; i++ ) add_to_table( tab_tex2[i], table4, i, 0, 0 );
+	for (i = 0; i < 8; i++) add_to_table(tab_tex2[i], table4, i, 0, 4);
 
+// !!! TODO: Change this into table-driven & see if code size decreases !!!
 	spinbutton_trans   = spin_to_table(table4, 0, 1, 4, mem_xpm_trans, -1, mem_cols - 1);
 	spinbutton_hotx    = spin_to_table(table4, 1, 1, 4, mem_xbm_hot_x, -1, mem_width - 1);
 	spinbutton_hoty    = spin_to_table(table4, 2, 1, 4, mem_xbm_hot_y, -1, mem_height - 1);
-	spinbutton_jpeg    = spin_to_table(table4, 3, 1, 4, mem_jpeg_quality, 0, 100);
-	spinbutton_recent  = spin_to_table(table4, 4, 1, 4, recent_files, 0, MAX_RECENT);
-	spinbutton_silence = spin_to_table(table4, 5, 1, 4, silence_limit, 0, 28);
+	spinbutton_jpeg    = spin_to_table(table4, 3, 1, 4, jpeg_quality, 0, 100);
+	spinbutton_jp2     = spin_to_table(table4, 4, 1, 4, jp2_rate, 1, 100);
+	spinbutton_png     = spin_to_table(table4, 5, 1, 4, png_compression, 0, 9);
+	spinbutton_recent  = spin_to_table(table4, 6, 1, 4, recent_files, 0, MAX_RECENT);
+	spinbutton_silence = spin_to_table(table4, 7, 1, 4, silence_limit, 0, 28);
+	checkbutton_tgaRLE = add_a_toggle(_("TGA RLE Compression"), page, tga_RLE);
+	checkbutton_tga565 = add_a_toggle(_("Read 16-bit TGAs as 5:6:5 BGR"), page, tga_565);
+	checkbutton_tgadef = add_a_toggle(_("Write TGAs in bottom-up row order"), page, tga_defdir);
+	checkbutton_undo   = add_a_toggle(_("Undoable image loading"), page, undo_load);
 
-//	add_hseparator( vbox_2, -2, 10 );
+///	---- TAB3 - PATHS
 
-	clipboard_entry = path_box(_("Clipboard Files"), vbox_2, FS_CLIP_FILE);
-	gtk_entry_set_text(GTK_ENTRY(clipboard_entry), mem_clip_file);
+	page = add_new_page(notebook1, _("Paths"));
 
-//	add_hseparator( vbox_2, -2, 10 );
+	clipboard_entry = mt_path_box(_("Clipboard Files"), page,
+		_("Select Clipboard File"), FS_CLIP_FILE);
+	gtkuncpy(txt, mem_clip_file, PATHTXT);
+	gtk_entry_set_text(GTK_ENTRY(clipboard_entry), txt);
 
-	entry_handbook[0] = path_box(_("HTML Browser Program"), vbox_2, FS_BROWSER_PROG);
-	gtk_entry_set_text(GTK_ENTRY(entry_handbook[0]),
-		inifile_get(HANDBOOK_BROWSER_INI, ""));
+	entry_handbook[0] = mt_path_box(_("HTML Browser Program"), page,
+		_("Select Browser Program"), FS_SELECT_FILE);
+	gtkuncpy(txt, inifile_get(HANDBOOK_BROWSER_INI, ""), PATHTXT);
+	gtk_entry_set_text(GTK_ENTRY(entry_handbook[0]), txt);
 
-	entry_handbook[1] = path_box(_("Location of Handbook index"), vbox_2,
-		FS_HANDBOOK_INDEX);
-	gtk_entry_set_text(GTK_ENTRY(entry_handbook[1]),
-		inifile_get(HANDBOOK_LOCATION_INI, ""));
+	entry_handbook[1] = mt_path_box(_("Location of Handbook index"), page,
+		_("Select Handbook Index File"), FS_SELECT_FILE);
+	gtkuncpy(txt, inifile_get(HANDBOOK_LOCATION_INI, ""), PATHTXT);
+	gtk_entry_set_text(GTK_ENTRY(entry_handbook[1]), txt);
 
-///	---- TAB3 - STATUS BAR
+	entry_def[0] = mt_path_box(_("Default Palette"), page,
+		_("Select Default Palette"), FS_SELECT_FILE);
+	gtkuncpy(txt, inifile_get(DEFAULT_PAL_INI, ""), PATHTXT);
+	gtk_entry_set_text(GTK_ENTRY(entry_def[0]), txt);
 
-	vbox_3 = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox_3);
-	gtk_container_add (GTK_CONTAINER (notebook1), vbox_3);
+	entry_def[1] = mt_path_box(_("Default Patterns"), page,
+		_("Select Default Patterns File"), FS_SELECT_FILE);
+	gtkuncpy(txt, inifile_get(DEFAULT_PAT_INI, ""), PATHTXT);
+	gtk_entry_set_text(GTK_ENTRY(entry_def[1]), txt);
 
-	label = gtk_label_new( _("Status Bar") );
-	gtk_widget_show (label);
-	gtk_notebook_set_tab_label(GTK_NOTEBOOK (notebook1),
-		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 2), label);
+
+///	---- TAB4 - STATUS BAR
+
+	page = add_new_page(notebook1, _("Status Bar"));
 
 	for ( i=0; i<STATUS_ITEMS; i++ )
 	{
-		sprintf(txt, "status%iToggle", i);
-		prefs_status[i] = add_a_toggle( stat_tex[i], vbox_3, inifile_get_gboolean(txt, TRUE) );
+		prefs_status[i] = add_a_toggle( stat_tex[i], page, status_on[i] );
 	}
 
-///	---- TAB4 - ZOOM
+///	---- TAB5 - ZOOM
 
-	vbox_3 = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox_3);
-	gtk_container_add (GTK_CONTAINER (notebook1), vbox_3);
-
-	label = gtk_label_new( _("Zoom") );
-	gtk_widget_show (label);
-	gtk_notebook_set_tab_label(GTK_NOTEBOOK (notebook1),
-		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 3), label);
-	table5 = add_a_table( 2, 4, 10, vbox_3 );
+	page = add_new_page(notebook1, _("Zoom"));
+	table5 = add_a_table( 2, 4, 10, page );
 
 ///	TABLE TEXT
 	for ( i=0; i<2; i++ ) add_to_table( tab_tex3[i], table5, i, 0, 5 );
@@ -590,48 +565,37 @@ void pressed_preferences( GtkMenuItem *menu_item, gpointer user_data )
 	}
 
 	checkbutton_zoom[0] = add_a_toggle( _("New image sets zoom to 100%"),
-		vbox_3, inifile_get_gboolean("zoomToggle", FALSE) );
+		page, inifile_get_gboolean("zoomToggle", FALSE) );
 #if GTK_MAJOR_VERSION == 2
 	checkbutton_zoom[1] = add_a_toggle( _("Mouse Scroll Wheel = Zoom"),
-		vbox_3, inifile_get_gboolean("scrollwheelZOOM", TRUE) );
+		page, inifile_get_gboolean("scrollwheelZOOM", FALSE) );
 #endif
 	checkbutton_zoom[2] = add_a_toggle( _("Optimize alpha chequers"),
-		vbox_3, chequers_optimize );
+		page, chequers_optimize );
 	checkbutton_zoom[3] = add_a_toggle( _("Disable view window transparencies"),
-		vbox_3, opaque_view );
+		page, opaque_view );
 
 
 
-///	---- TAB5 - TABLET
-
-	vbox_3 = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox_3);
-	gtk_container_add (GTK_CONTAINER (notebook1), vbox_3);
-
-	label = gtk_label_new( _("Tablet") );
-	gtk_widget_show (label);
-	gtk_notebook_set_tab_label(GTK_NOTEBOOK (notebook1),
-		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 4), label);
-
+///	---- TAB6 - TABLET
 
+	page = add_new_page(notebook1, _("Tablet"));
 
 	vbox_2 = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox_2);
-	add_with_frame(vbox_3, _("Device Settings"), vbox_2, 5);
+	add_with_frame(page, _("Device Settings"), vbox_2, 5);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox_2), 5);
 
-	label_tablet_device = gtk_label_new ("");
+	label_tablet_device = pack(vbox_2, gtk_label_new(""));
 	gtk_widget_show (label_tablet_device);
-	gtk_box_pack_start (GTK_BOX (vbox_2), label_tablet_device, FALSE, FALSE, 0);
 	gtk_misc_set_alignment (GTK_MISC (label_tablet_device), 0, 0.5);
 	gtk_misc_set_padding (GTK_MISC (label_tablet_device), 5, 5);
 
 	button1 = add_a_button( _("Configure Device"), 0, vbox_2, FALSE );
 	gtk_signal_connect(GTK_OBJECT(button1), "clicked", GTK_SIGNAL_FUNC(conf_tablet), NULL);
 
-	table3 = gtk_table_new (4, 2, FALSE);
+	table3 = xpack(vbox_2, gtk_table_new(4, 2, FALSE));
 	gtk_widget_show (table3);
-	gtk_box_pack_start (GTK_BOX (vbox_2), table3, TRUE, TRUE, 0);
 
 	label = add_to_table( _("Tool Variable"), table3, 0, 0, 0 );
 	gtk_misc_set_padding (GTK_MISC (label), 5, 5);
@@ -663,12 +627,11 @@ void pressed_preferences( GtkMenuItem *menu_item, gpointer user_data )
 
 	vbox_2 = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox_2);
-	add_with_frame(vbox_3, _("Test Area"), vbox_2, 5);
+	add_with_frame(page, _("Test Area"), vbox_2, 5);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox_2), 5);
 
-	drawingarea_tablet = gtk_drawing_area_new ();
+	drawingarea_tablet = xpack(vbox_2, gtk_drawing_area_new());
 	gtk_widget_show (drawingarea_tablet);
-	gtk_box_pack_start (GTK_BOX (vbox_2), drawingarea_tablet, TRUE, TRUE, 0);
 	gtk_widget_set_usize (drawingarea_tablet, 128, 64);
 	gtk_signal_connect( GTK_OBJECT(drawingarea_tablet), "expose_event",
 		GTK_SIGNAL_FUNC (expose_tablet_preview), (gpointer) drawingarea_tablet );
@@ -688,41 +651,32 @@ void pressed_preferences( GtkMenuItem *menu_item, gpointer user_data )
 
 
 
-	label_tablet_pressure = gtk_label_new ("");
+	label_tablet_pressure = pack(vbox_2, gtk_label_new(""));
 	gtk_widget_show (label_tablet_pressure);
-	gtk_box_pack_start (GTK_BOX (vbox_2), label_tablet_pressure, FALSE, FALSE, 0);
 	gtk_misc_set_alignment (GTK_MISC (label_tablet_pressure), 0, 0.5);
 
 
 
-///	---- TAB6 - LANGUAGE
+///	---- TAB7 - LANGUAGE
 
 #ifdef U_NLS
 
-	vbox_2 = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox_2);
-	gtk_container_add (GTK_CONTAINER (notebook1), vbox_2);
-	gtk_container_set_border_width( GTK_CONTAINER(vbox_2), 10 );
+	page = add_new_page(notebook1, _("Language"));
+	gtk_container_set_border_width(GTK_CONTAINER(page), 10);
 
-	label = gtk_label_new( _("Language") );
+	add_hseparator( page, 200, 10 );
+	label = pack(page, gtk_label_new( _("Select preferred language translation\n\n"
+				"You will need to restart mtPaint\nfor this to take full effect")));
 	gtk_widget_show (label);
-	gtk_notebook_set_tab_label(GTK_NOTEBOOK (notebook1),
-		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 5), label);
-
-	add_hseparator( vbox_2, 200, 10 );
-	label = gtk_label_new( _("Select preferred language translation\n\n"
-				"You will need to restart mtPaint\nfor this to take full effect") );
-	gtk_widget_show (label);
-	gtk_box_pack_start( GTK_BOX(vbox_2), label, FALSE, FALSE, 0 );
-	add_hseparator( vbox_2, 200, 10 );
+	add_hseparator( page, 200, 10 );
 
 	for (i = 0; i < PREF_LANGS; i++)
 	{
 		if (!strcmp(pref_lang_ini_code[i],
 			inifile_get("languageSETTING", "system"))) break;
 	}
-	hbox4 = wj_radio_pack(pref_langs, PREF_LANGS, 6, i, &pref_lang, NULL);
-	gtk_box_pack_start(GTK_BOX(vbox_2), hbox4, TRUE, TRUE, 0);
+	xpack(page, wj_radio_pack(pref_langs, PREF_LANGS, (PREF_LANGS + 1) / 2,
+		i, &pref_lang, NULL));
 
 #endif
 
@@ -730,10 +684,9 @@ void pressed_preferences( GtkMenuItem *menu_item, gpointer user_data )
 
 ///	Bottom of Prefs window
 
-	hbox4 = OK_box(0, prefs_window, _("OK"), GTK_SIGNAL_FUNC(prefs_ok),
-		_("Cancel"), GTK_SIGNAL_FUNC(delete_prefs));
+	hbox4 = pack(vbox3, OK_box(0, prefs_window, _("OK"), GTK_SIGNAL_FUNC(prefs_ok),
+		_("Cancel"), GTK_SIGNAL_FUNC(delete_prefs)));
 	OK_box_add(hbox4, _("Apply"), GTK_SIGNAL_FUNC(prefs_apply), 1);
-	gtk_box_pack_start (GTK_BOX (vbox3), hbox4, FALSE, FALSE, 0);
 
 	gtk_window_set_transient_for( GTK_WINDOW(prefs_window), GTK_WINDOW(main_window) );
 	gtk_widget_show (prefs_window);
@@ -748,9 +701,8 @@ void pressed_preferences( GtkMenuItem *menu_item, gpointer user_data )
 void init_tablet()				// Set up variables
 {
 	int i;
-	char *devname, txt[32];
+	char *devname, txt[64];
 
-	gboolean use_tablet;
 	GList *dlist;
 
 #if GTK_MAJOR_VERSION == 1
@@ -762,11 +714,10 @@ void init_tablet()				// Set up variables
 	GdkAxisUse use;
 #endif
 
-	use_tablet = inifile_get_gboolean( "tablet_USE", FALSE );
-
-	if ( use_tablet )		// User has got tablet working in past so try to initialize it
+	if (tablet_working)	// User has got tablet working in past so try to initialize it
 	{
-		devname = inifile_get( tablet_ini3[0], "?" );	// Device name last used
+		tablet_working = FALSE;
+		devname = inifile_get( "tablet_name", "?" );	// Device name last used
 #if GTK_MAJOR_VERSION == 1
 		dlist = gdk_input_list_devices();
 #endif
@@ -781,14 +732,15 @@ void init_tablet()				// Set up variables
 
 #if GTK_MAJOR_VERSION == 1
 				gdk_input_set_mode(device->deviceid,
-					inifile_get_gint32( tablet_ini3[1], 0 ) );
+					inifile_get_gint32("tablet_mode", 0));
 #endif
 #if GTK_MAJOR_VERSION == 2
-				gdk_device_set_mode(device, inifile_get_gint32( tablet_ini3[1], 0 ) );
+				gdk_device_set_mode(device,
+					inifile_get_gint32("tablet_mode", 0));
 #endif
 				for ( i=0; i<device->num_axes; i++ )
 				{
-					sprintf(txt, "%s%i", tablet_ini3[2], i);
+					sprintf(txt, "tablet_axes_v%i", i);
 					use = inifile_get_gint32( txt, GDK_AXIS_IGNORE );
 #if GTK_MAJOR_VERSION == 1
 					device->axes[i] = use;
