@@ -1,11 +1,11 @@
 /*	toolbar.c
-	Copyright (C) 2006-2007 Mark Tyler and Dmitry Groshev
+	Copyright (C) 2006-2008 Mark Tyler and Dmitry Groshev
 
 	This file is part of mtPaint.
 
 	mtPaint is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 2 of the License, or
+	the Free Software Foundation; either version 3 of the License, or
 	(at your option) any later version.
 
 	mtPaint is distributed in the hope that it will be useful,
@@ -17,24 +17,21 @@
 	along with mtPaint in the file COPYING.
 */
 
-#include <gtk/gtk.h>
-#include <math.h>
-
-
 #include "global.h"
 
+#include "mygtk.h"
 #include "memory.h"
 #include "inifile.h"
 #include "png.h"
 #include "mainwindow.h"
 #include "otherwindow.h"
 #include "canvas.h"
-#include "mygtk.h"
 #include "toolbar.h"
 #include "layer.h"
 #include "viewer.h"
 #include "channels.h"
 #include "csel.h"
+#include "font.h"
 
 
 #include "graphics/xpm_paint.xpm"
@@ -124,6 +121,7 @@
 #include "graphics/xpm_mode_tint.xpm"
 #include "graphics/xpm_mode_tint2.xpm"
 #include "graphics/xpm_mode_csel.xpm"
+#include "graphics/xpm_mode_blend.xpm"
 #include "graphics/xpm_mode_mask.xpm"
 
 #include "graphics/xpm_grad_place.xpm"
@@ -134,69 +132,70 @@
 GtkWidget *icon_buttons[TOTAL_ICONS_TOOLS];
 
 gboolean toolbar_status[TOOLBAR_MAX];			// True=show
-GtkWidget *toolbar_boxes[TOOLBAR_MAX]			// Used for showing/hiding
-		= {NULL, NULL, NULL, NULL, NULL, NULL},
-	*toolbar_menu_widgets[TOOLBAR_MAX],		// Menu widgets
-	*drawing_col_prev = NULL;
+GtkWidget *toolbar_boxes[TOOLBAR_MAX],			// Used for showing/hiding
+	*drawing_col_prev;
 
 GdkCursor *move_cursor;
 GdkCursor *m_cursor[32];		// My mouse cursors
 
 
 
-static GtkWidget *toolbar_zoom_main = NULL, *toolbar_zoom_view,
+static GtkWidget *toolbar_zoom_main, *toolbar_zoom_view,
 	*toolbar_labels[2],		// Colour A & B details
 	*ts_spinslides[3],		// Size, flow, opacity
 	*tb_label_opacity		// Opacity label
 	;
-static unsigned char *mem_prev = NULL;		// RGB colours, tool, pattern preview
+static unsigned char mem_prev[PREVIEW_WIDTH * PREVIEW_HEIGHT * 3];
+					// RGB colours, tool, pattern preview
 
+typedef struct
+{
+	unsigned char ID;
+	signed char radio;
+	unsigned char sep, rclick;
+	int actmap;
+	char *tooltip, **xpm;
+	GtkWidget *widget;
+} toolbar_item;
 
-GtkWidget *layer_iconbar(GtkWidget *window, GtkWidget *box, GtkWidget **icons)
-{		// Create iconbar for layers window
-	char **icon_list[7] = {
-		xpm_new_xpm, xpm_up_xpm, xpm_down_xpm, xpm_copy_xpm, xpm_centre_xpm,
-		xpm_cut_xpm, xpm_close_xpm
-		};
+static void fill_toolbar(GtkToolbar *bar, toolbar_item *items,
+	GtkSignalFunc lclick, int lbase, GtkSignalFunc rclick, int rbase);
 
-	char *hint_text[7] = {
-		_("New Layer"), _("Raise"), _("Lower"), _("Duplicate Layer"), _("Centralise Layer"),
-		_("Delete Layer"), _("Close Layers Window")
-		};
+#undef _
+#define _(X) X
 
-	gint i, offset[7] = {3, 1, 2, 4, 6, 5, 8};//0, 0, 7, 8};
+static toolbar_item layer_bar[] = {
+	{ LTB_NEW,    -1, 0, 0, 0, _("New Layer"), xpm_new_xpm },
+	{ LTB_RAISE,  -1, 0, 0, 0, _("Raise"), xpm_up_xpm },
+	{ LTB_LOWER,  -1, 0, 0, 0, _("Lower"), xpm_down_xpm },
+	{ LTB_DUP,    -1, 0, 0, 0, _("Duplicate Layer"), xpm_copy_xpm },
+	{ LTB_CENTER, -1, 0, 0, 0, _("Centralise Layer"), xpm_centre_xpm },
+	{ LTB_DEL,    -1, 0, 0, 0, _("Delete Layer"), xpm_cut_xpm },
+	{ LTB_CLOSE,  -1, 0, 0, 0, _("Close Layers Window"), xpm_close_xpm },
+	{ 0, 0, 0, 0, 0, NULL, NULL }};
 
-	GtkWidget *toolbar, *iconw;
-	GdkPixmap *icon, *mask;
+#undef _
+#define _(X) __(X)
 
-// we need to realize the window because we use pixmaps for 
-// items on the toolbar in the context of it
-	gtk_widget_realize( window );
+/* Create toolbar for layers window */
+GtkWidget *layer_toolbar(GtkWidget **wlist)
+{		
+	int i;
+	GtkWidget *toolbar;
 
 #if GTK_MAJOR_VERSION == 1
-	toolbar = gtk_toolbar_new( GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS );
+	toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
 #endif
 #if GTK_MAJOR_VERSION == 2
 	toolbar = gtk_toolbar_new();
-	gtk_toolbar_set_style( GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS );
+	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
 #endif
+	fill_toolbar(GTK_TOOLBAR(toolbar), layer_bar,
+		GTK_SIGNAL_FUNC(layer_iconbar_click), 0, NULL, 0);
+	gtk_widget_show(toolbar);
 
-	gtk_box_pack_start ( GTK_BOX (box), toolbar, FALSE, FALSE, 0 );
-
-	for (i=0; i<7; i++)
-	{
-		icon = gdk_pixmap_create_from_xpm_d ( main_window->window, &mask,
-			NULL, icon_list[i] );
-		iconw = gtk_pixmap_new ( icon, mask );
-		gdk_pixmap_unref( icon );
-		gdk_pixmap_unref( mask );
-
-		icons[ offset[i] ] =
-			gtk_toolbar_append_element( GTK_TOOLBAR(toolbar),
-			GTK_TOOLBAR_CHILD_BUTTON, NULL, "None", hint_text[i],
-			"Private", iconw, GTK_SIGNAL_FUNC(layer_iconbar_click), (gpointer) i);
-	}
-	gtk_widget_show ( toolbar );
+	for (i = 0; i < TOTAL_ICONS_LAYER; i++)
+		wlist[i] = layer_bar[i].widget;
 
 	return toolbar;
 }
@@ -252,10 +251,9 @@ static GtkWidget *toolbar_add_zoom(GtkWidget *box)		// Add zoom combo box
 	GList *combo_list = NULL;
 
 
-	combo = gtk_combo_new ();
+	combo = pack(box, gtk_combo_new());
 	gtk_combo_set_value_in_list (GTK_COMBO (combo), FALSE, FALSE);
 	gtk_widget_show (combo);
-	gtk_box_pack_start (GTK_BOX (box), combo, FALSE, FALSE, 0);
 	combo_entry = GTK_COMBO (combo)->entry;
 	GTK_WIDGET_UNSET_FLAGS (combo_entry, GTK_CAN_FOCUS);
 	gtk_widget_set_usize(GTK_COMBO(combo)->button, 18, -1);
@@ -288,12 +286,8 @@ static float toolbar_get_zoom( GtkWidget *combo )
 	if ( strlen(txt) > 2)		// Weed out bogus calls
 	{
 		sscanf(txt, "%i%%", &i);
-		res = ((float) i ) / 100;
-		if ( res<1 )
-		{
-			res = mt_round( 1/res );
-			res = 1/res;
-		}
+		res = (float)i / 100;
+		if (res < 1.0) res = 1.0 / rint(1.0 / res);
 	}
 
 	return res;
@@ -338,7 +332,8 @@ static void toolbar_zoom_view_change()
 static int *vars_settings[TOTAL_SETTINGS] = {
 	&mem_continuous, &mem_undo_opacity,
 	&tint_mode[0], &tint_mode[1],
-	&mem_cselect, &mem_unmask, &mem_gradient
+	&mem_cselect, &mem_blend,
+	&mem_unmask, &mem_gradient
 };
 
 void toolbar_mode_change(GtkWidget *widget, gpointer data)
@@ -349,12 +344,6 @@ void toolbar_mode_change(GtkWidget *widget, gpointer data)
 
 	switch (j)
 	{
-	case SETB_CONT:
-		inifile_set_gboolean( "continuousPainting", mem_continuous );
-		return;
-	case SETB_OPAC:
-		inifile_set_gboolean( "opacityToggle", mem_undo_opacity );
-		return;
 	case SETB_CSEL:
 		if (mem_cselect && !csel_data)
 		{
@@ -377,8 +366,7 @@ static int set_flood(GtkWidget *box, gpointer fdata)
 	GList *chain = GTK_BOX(box)->children;
 
 	spin = ((GtkBoxChild*)chain->data)->widget;
-	gtk_spin_button_update(GTK_SPIN_BUTTON(spin));
-	flood_step = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spin));
+	flood_step = read_float_spin(spin);
 	chain = chain->next;
 	toggle = ((GtkBoxChild*)chain->data)->widget;
 	flood_cube = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle));
@@ -396,16 +384,49 @@ static int set_smudge(GtkWidget *box, gpointer fdata)
 {
 	GtkWidget *toggle;
 
-	toggle = ((GtkBoxChild*)GTK_BOX(box)->children->data)->widget;
+	toggle = BOX_CHILD_0(box);
 	smudge_mode = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle));
 
 	return TRUE;
 }
 
+static int set_brush_step(GtkWidget *box, gpointer fdata)
+{
+	GtkWidget *spin;
+
+	spin = BOX_CHILD_0(box);
+	brush_spacing = read_spin(spin);
+
+	return TRUE;
+}
+
+static int blendtemp[5];
+
+static int set_blend(GtkWidget *box, gpointer fdata)
+{
+	int i, j;
+
+	i = blendtemp[0] < 0 ? BLEND_NORMAL : blendtemp[0];
+	j = !blendtemp[2] + (!blendtemp[3] << 1) + (!blendtemp[4] << 2);
+
+	/* Don't accept stop-all or do-nothing */
+	if ((j == 7) || !(i | j)) return (FALSE);
+
+	blend_mode = i | (blendtemp[1] ? BLEND_REVERSE : 0) |
+		(j << BLEND_RGBSHIFT);
+
+	return (TRUE);
+}
+
 static gboolean toolbar_rclick(GtkWidget *widget, GdkEventButton *event,
 	gpointer user_data)
 {
-	GtkWidget *spin, *box;
+	char *rgbnames[3] = { _("Red"), _("Green"), _("Blue") };
+	char *blends[BLEND_NMODES] = {
+		_("Normal"), _("Hue"), _("Saturation"), _("Value"),
+		_("Colour"), _("Saturate More") };
+	GtkWidget *box, *hbox;
+	int i, k;
 
 	/* Handle only right clicks */
 	if ((event->type != GDK_BUTTON_PRESS) || (event->button != 3))
@@ -413,19 +434,43 @@ static gboolean toolbar_rclick(GtkWidget *widget, GdkEventButton *event,
 
 	switch ((gint)user_data)
 	{
+	case SETB_CONT: /* Brush spacing */
+		box = gtk_vbox_new(FALSE, 5);
+		gtk_widget_show(box);
+		pack(box, add_a_spin(brush_spacing, 0, MAX_WIDTH));
+// !!! Not implemented yet
+//		add_a_toggle(_("Flat gradient strokes"), box, ???);
+		filter_window(_("Brush spacing"), box, set_brush_step, NULL, TRUE);
+		break;
 	case SETB_CSEL:
 		colour_selector(COLSEL_EDIT_CSEL);
 		break;
 	case SETB_GRAD: /* Gradient selector */
 		gradient_setup(1);
 		break;
+	case SETB_FILT: /* Blend mode */
+		box = gtk_vbox_new(FALSE, 5);
+		gtk_container_set_border_width(GTK_CONTAINER(box), 5);
+		k = 0;
+		pack(box, wj_combo_box(blends, BLEND_NMODES,
+			blend_mode & BLEND_MMASK, blendtemp + 0, NULL));
+		pack(box, sig_toggle(_("Reverse"), blend_mode & BLEND_REVERSE,
+			blendtemp + 1, NULL));
+		add_hseparator(box, -2, 10);
+		hbox = pack(box, gtk_hbox_new(TRUE, 5));
+		for (i = 0; i < 3; i++)
+		{
+			pack(hbox, sig_toggle(rgbnames[i],
+				~blend_mode & ((1 << BLEND_RGBSHIFT) << i),
+				blendtemp + 2 + i, NULL));
+		}
+		gtk_widget_show_all(box);
+		filter_window(_("Blend mode"), box, set_blend, NULL, TRUE);
+		break;
 	case (TTB_0 + TTB_FLOOD): /* Flood fill step */
-		spin = add_a_spin(0, 0, 200);
-		gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 2);
-		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), flood_step);
 		box = gtk_vbox_new(FALSE, 5);
 		gtk_widget_show(box);
-		gtk_box_pack_start(GTK_BOX(box), spin, FALSE, FALSE, 0);
+		pack(box, add_float_spin(flood_step, 0, 200));
 		add_a_toggle(_("RGB Cube"), box, flood_cube);
 		add_a_toggle(_("By image channel"), box, flood_img);
 		add_a_toggle(_("Gradient-driven"), box, flood_slide);
@@ -440,6 +485,9 @@ static gboolean toolbar_rclick(GtkWidget *widget, GdkEventButton *event,
 	case (TTB_0 + TTB_GRAD): /* Gradient config */
 		gradient_setup(0);
 		break;
+	case (TTB_0 + TTB_TEXT): /* FreeType text paste */
+		pressed_mt_text(NULL, NULL);
+		break;
 	default: /* For other buttons, do nothing */
 		return (FALSE);
 	}
@@ -478,18 +526,11 @@ static void ts_spinslide_moved(GtkAdjustment *adj, gpointer user_data)
 static void toolbar_settings_exit()
 {
 	toolbar_status[TOOLBAR_SETTINGS] = FALSE;
-	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(toolbar_menu_widgets[TOOLBAR_SETTINGS]),
-			FALSE );
+	gtk_check_menu_item_set_active(
+		GTK_CHECK_MENU_ITEM(menu_widgets[MENU_TBSET]), FALSE);
 	toolbar_exit();
 }
 
-typedef struct
-{
-	int ID, radio, sep, rclick, actmap;
-	char *tooltip, **xpm;
-	GtkWidget *widget;
-} toolbar_item;
-
 static void fill_toolbar(GtkToolbar *bar, toolbar_item *items,
 	GtkSignalFunc lclick, int lbase, GtkSignalFunc rclick, int rbase)
 {
@@ -515,50 +556,7 @@ static void fill_toolbar(GtkToolbar *bar, toolbar_item *items,
 		if (items->rclick) gtk_signal_connect(GTK_OBJECT(items->widget),
 			"button_press_event", rclick, (gpointer)(items->ID + rbase));
 		if (items->sep) gtk_toolbar_append_space(bar);
-	}
-}
-
-#define NEED_UNDO  0x0001
-#define NEED_REDO  0x0002
-#define NEED_CROP  0x0004
-#define NEED_MARQ  0x0008
-#define NEED_SEL   0x0010
-#define NEED_CLIP  0x0020
-#define NEED_HELP  0x0040
-#define NEED_24    0x0080
-#define NEED_IDX   0x0100
-#define NEED_CLINE 0x0200
-#define NEED_LAYER 0x0400
-#define NEED_LASSO 0x0800
-#define NEED_PREFS 0x1000
-#define NEED_ALPHA 0x2000
-#define NEED_CHAN  0x4000
-#define NEED_SEL2  (NEED_SEL | NEED_LASSO)
-
-static GtkWidget **need_lists[] = {
-	menu_undo, menu_redo, menu_crop, menu_need_marquee,
-	menu_need_selection, menu_need_clipboard, menu_help, menu_only_24,
-	menu_only_indexed, menu_cline, menu_layer, menu_lasso, menu_prefs,
-	menu_alphablend, menu_chan_del };
-
-static void tool_dis_add(toolbar_item *items)
-{
-	int i, j;
-
-	for (; items->xpm; items++)
-	{
-		if (!items->actmap) continue;
-		i = items->actmap;
-		while (i)
-		{
-			j = i; i &= i - 1; j = (j ^ i) - 1;
-			j = (j & 0x55555555) + ((j >> 1) & 0x55555555);
-			j = (j & 0x33333333) + ((j >> 2) & 0x33333333);
-			j = (j & 0x0F0F0F0F) + ((j >> 4) & 0x0F0F0F0F);
-			j = (j & 0x00FF00FF) + ((j >> 8) & 0x00FF00FF);
-			j = (j & 0xFFFF) + (j >> 16);
-			men_dis_add(items->widget, need_lists[j]);
-		}
+		mapped_dis_add(items->widget, items->actmap);
 	}
 }
 
@@ -570,11 +568,12 @@ static GtkWidget *grad_view;
 #define _(X) X
 
 static toolbar_item settings_bar[] = {
-	{ SETB_CONT, 0, 0, 0, 0, _("Continuous Mode"), xpm_mode_cont_xpm },
+	{ SETB_CONT, 0, 0, 1, 0, _("Continuous Mode"), xpm_mode_cont_xpm },
 	{ SETB_OPAC, 0, 0, 0, 0, _("Opacity Mode"), xpm_mode_opac_xpm },
 	{ SETB_TINT, 0, 0, 0, 0, _("Tint Mode"), xpm_mode_tint_xpm },
 	{ SETB_TSUB, 0, 0, 0, 0, _("Tint +-"), xpm_mode_tint2_xpm },
 	{ SETB_CSEL, 0, 0, 1, 0, _("Colour-Selective Mode"), xpm_mode_csel_xpm },
+	{ SETB_FILT, 0, 0, 1, 0, _("Blend Mode"), xpm_mode_blend_xpm },
 	{ SETB_MASK, 0, 0, 0, 0, _("Disable All Masks"), xpm_mode_mask_xpm },
 	{ 0, 0, 0, 0, 0, NULL, NULL }};
 
@@ -595,19 +594,32 @@ static void toolbar_settings_init()
 	{
 		gtk_widget_show( toolbar_boxes[TOOLBAR_SETTINGS] );	// Used when Home key is pressed
 #if GTK_MAJOR_VERSION == 1
-		gtk_widget_queue_resize(toolbar_boxes[TOOLBAR_SETTINGS]); /* Reset shortened sliders */
+		gtk_widget_queue_resize(toolbar_boxes[TOOLBAR_SETTINGS]); /* Re-render sliders */
 #endif
 		return;
 	}
 
 ///	SETTINGS TOOLBAR
 
+	toolbar_boxes[TOOLBAR_SETTINGS] = add_a_window(GTK_WINDOW_TOPLEVEL,
+		_("Settings Toolbar"), GTK_WIN_POS_NONE, FALSE);
+
+	gtk_widget_set_uposition( toolbar_boxes[TOOLBAR_SETTINGS],
+		inifile_get_gint32("toolbar_settings_x", 0 ),
+		inifile_get_gint32("toolbar_settings_y", 0 ) );
+
+	vbox = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show(vbox);
+	gtk_container_add (GTK_CONTAINER (toolbar_boxes[TOOLBAR_SETTINGS]), vbox);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
+
 #if GTK_MAJOR_VERSION == 1
-	toolbar_settings = gtk_toolbar_new( GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS );
+	toolbar_settings = pack(vbox, gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,
+		GTK_TOOLBAR_ICONS));
 #endif
 #if GTK_MAJOR_VERSION == 2
-	toolbar_settings = gtk_toolbar_new();
-	gtk_toolbar_set_style( GTK_TOOLBAR(toolbar_settings), GTK_TOOLBAR_ICONS );
+	toolbar_settings = pack(vbox, gtk_toolbar_new());
+	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar_settings), GTK_TOOLBAR_ICONS);
 #endif
 
 	fill_toolbar(GTK_TOOLBAR(toolbar_settings), settings_bar,
@@ -620,24 +632,10 @@ static void toolbar_settings_init()
 			!!*(vars_settings[i]));
 	}
 
-	toolbar_boxes[TOOLBAR_SETTINGS] = add_a_window( GTK_WINDOW_TOPLEVEL, _("Settings Toolbar"),
-			GTK_WIN_POS_NONE, FALSE );
-
-	gtk_widget_set_uposition( toolbar_boxes[TOOLBAR_SETTINGS],
-		inifile_get_gint32("toolbar_settings_x", 0 ),
-		inifile_get_gint32("toolbar_settings_y", 0 ) );
-
-	vbox = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show(vbox);
-	gtk_container_add (GTK_CONTAINER (toolbar_boxes[TOOLBAR_SETTINGS]), vbox);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
-
-	gtk_box_pack_start ( GTK_BOX (vbox), toolbar_settings, FALSE, FALSE, 0 );
 	gtk_widget_show(toolbar_settings);
 
 	/* Gradient mode button+preview */
-	button = gtk_toggle_button_new();
-	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
+	button = pack(vbox, gtk_toggle_button_new());
 	pmap = gdk_pixmap_new(main_window->window, GP_WIDTH, GP_HEIGHT, -1);
 	grad_view = gtk_pixmap_new(pmap, NULL);
 	gdk_pixmap_unref(pmap);
@@ -658,19 +656,17 @@ static void toolbar_settings_init()
 		button, _("Gradient Mode"), "Private");
 
 	/* Colors A & B */
-	label = gtk_label_new("");
+	label = pack(vbox, gtk_label_new(""));
 	toolbar_labels[0] = label;
 	gtk_misc_set_alignment( GTK_MISC(label), 0, 0.5 );
 	gtk_widget_show (label);
 	gtk_misc_set_padding (GTK_MISC (label), 5, 2);
-	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
 
-	label = gtk_label_new("");
+	label = pack(vbox, gtk_label_new(""));
 	toolbar_labels[1] = label;
 	gtk_misc_set_alignment( GTK_MISC(label), 0, 0.5 );
 	gtk_widget_show (label);
 	gtk_misc_set_padding (GTK_MISC (label), 5, 2);
-	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
 
 	if (mem_channel != CHN_IMAGE) vals[2] = channel_col_A[mem_channel];
 	table = add_a_table(3, 2, 5, vbox);
@@ -723,7 +719,7 @@ static toolbar_item tools_bar[] = {
 	{ TTB_POLY, 1, 0, 0, 0, _("Polygon Selection"), xpm_polygon_xpm },
 	{ TTB_GRAD, 1, 1, 1, 0, _("Place Gradient"), xpm_grad_place_xpm },
 	{ TTB_LASSO, -1, 0, 0, NEED_LASSO, _("Lasso Selection"), xpm_lasso_xpm },
-	{ TTB_TEXT, -1, 1, 0, 0, _("Paste Text"), xpm_text_xpm },
+	{ TTB_TEXT, -1, 1, 1, 0, _("Paste Text"), xpm_text_xpm },
 	{ TTB_ELLIPSE, -1, 0, 0, NEED_SEL, _("Ellipse Outline"), xpm_ellipse2_xpm },
 	{ TTB_FELLIPSE, -1, 0, 0, NEED_SEL, _("Filled Ellipse"), xpm_ellipse_xpm },
 	{ TTB_OUTLINE, -1, 0, 0, NEED_SEL2, _("Outline Selection"), xpm_rect1_xpm },
@@ -767,31 +763,25 @@ void toolbar_init(GtkWidget *vbox_main)
 		sprintf(txt, "toolbar%i", i);
 		toolbar_status[i] = inifile_get_gboolean( txt, TRUE );
 
-		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(toolbar_menu_widgets[i]),
-				toolbar_status[i] );	// Menu toggles = status
+		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
+			menu_widgets[MENU_TBMAIN - TOOLBAR_MAIN + i]),
+			toolbar_status[i]);	// Menu toggles = status
 	}
 
 ///	MAIN TOOLBAR
 
-	hbox = gtk_hbox_new (FALSE, 0);
-	toolbar_boxes[TOOLBAR_MAIN] = hbox;
-	if ( toolbar_status[TOOLBAR_MAIN] ) gtk_widget_show (hbox);	// Only show if user wants
-	gtk_box_pack_start ( GTK_BOX (vbox_main), hbox, FALSE, FALSE, 0 );
-
+	toolbar_boxes[TOOLBAR_MAIN] = hbox = pack(vbox_main, gtk_hbox_new(FALSE, 0));
+	if (toolbar_status[TOOLBAR_MAIN]) gtk_widget_show(hbox); // Only show if user wants
 
 #if GTK_MAJOR_VERSION == 1
-	toolbar_main = gtk_toolbar_new( GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS );
-	toolbar_tools = gtk_toolbar_new( GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS );
+	toolbar_main = pack(hbox, gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,
+		GTK_TOOLBAR_ICONS));
 #endif
 #if GTK_MAJOR_VERSION == 2
-	toolbar_main = gtk_toolbar_new();
+	toolbar_main = pack(hbox, gtk_toolbar_new());
 	gtk_toolbar_set_style( GTK_TOOLBAR(toolbar_main), GTK_TOOLBAR_ICONS );
-	toolbar_tools = gtk_toolbar_new();
-	gtk_toolbar_set_style( GTK_TOOLBAR(toolbar_tools), GTK_TOOLBAR_ICONS );
 #endif
 
-	gtk_box_pack_start ( GTK_BOX (hbox), toolbar_main, FALSE, FALSE, 0 );
-
 	toolbar_zoom_main = toolbar_add_zoom(hbox);
 	toolbar_zoom_view = toolbar_add_zoom(hbox);
 	/* In GTK1, combo box entry is updated continuously */
@@ -826,33 +816,36 @@ void toolbar_init(GtkWidget *vbox_main)
 	gdk_pixmap_unref( icon );
 	gdk_pixmap_unref( mask );
 
-	fill_toolbar(GTK_TOOLBAR(toolbar_tools), tools_bar,
-		GTK_SIGNAL_FUNC(toolbar_icon_event), 0,
-		GTK_SIGNAL_FUNC(toolbar_rclick), TTB_0);
-	tool_dis_add(tools_bar);
-	for (i = 0; tools_bar[i].xpm; i++)
-	{
-		icon_buttons[tools_bar[i].ID] = tools_bar[i].widget;
-	}
-
 	fill_toolbar(GTK_TOOLBAR(toolbar_main), main_bar,
 		GTK_SIGNAL_FUNC(toolbar_icon_event2), 0, NULL, 0);
-	tool_dis_add(main_bar);
-
-	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(icon_buttons[PAINT_TOOL_ICON]), TRUE );
-	gtk_widget_show ( toolbar_main );
 
+	gtk_widget_show(toolbar_main);
 
 
 ///	TOOLS TOOLBAR
 
-	hbox = gtk_hbox_new (FALSE, 0);
-	toolbar_boxes[TOOLBAR_TOOLS] = hbox;
-	if ( toolbar_status[TOOLBAR_TOOLS] ) gtk_widget_show (hbox);	// Only show if user wants
-	gtk_box_pack_start ( GTK_BOX (vbox_main), hbox, FALSE, FALSE, 0 );
+	toolbar_boxes[TOOLBAR_TOOLS] = hbox = pack(vbox_main, gtk_hbox_new(FALSE, 0));
+	if (toolbar_status[TOOLBAR_TOOLS]) gtk_widget_show(hbox); // Only show if user wants
 
-	gtk_box_pack_start ( GTK_BOX (hbox), toolbar_tools, FALSE, FALSE, 0 );
-	gtk_widget_show ( toolbar_tools );
+#if GTK_MAJOR_VERSION == 1
+	toolbar_tools = pack(hbox, gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,
+		GTK_TOOLBAR_ICONS));
+#endif
+#if GTK_MAJOR_VERSION == 2
+	toolbar_tools = pack(hbox, gtk_toolbar_new());
+	gtk_toolbar_set_style( GTK_TOOLBAR(toolbar_tools), GTK_TOOLBAR_ICONS );
+#endif
+
+	fill_toolbar(GTK_TOOLBAR(toolbar_tools), tools_bar,
+		GTK_SIGNAL_FUNC(toolbar_icon_event), 0,
+		GTK_SIGNAL_FUNC(toolbar_rclick), TTB_0);
+	for (i = 0; tools_bar[i].xpm; i++)
+	{
+		icon_buttons[tools_bar[i].ID] = tools_bar[i].widget;
+	}
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icon_buttons[PAINT_TOOL_ICON]), TRUE);
+
+	gtk_widget_show(toolbar_tools);
 }
 
 void ts_update_gradient()
@@ -961,30 +954,44 @@ void toolbar_update_settings()
 		gtk_label_set_text( GTK_LABEL(tb_label_opacity), channames[mem_channel] );
 }
 
-static gint expose_palette( GtkWidget *widget, GdkEventExpose *event )
+static gboolean expose_palette(GtkWidget *widget, GdkEventExpose *event,
+	gpointer user_data)
 {
-	gdk_draw_rgb_image( widget->window, widget->style->black_gc,
-				event->area.x, event->area.y, event->area.width, event->area.height,
-				GDK_RGB_DITHER_NONE,
-				mem_pals + 3*( event->area.x + PALETTE_WIDTH*event->area.y ),
-				PALETTE_WIDTH*3
-				);
+	int x2 = event->area.x + event->area.width;
+	int y2 = event->area.y + event->area.height;
 
-	return FALSE;
+	/* With theme engines lurking out there, weirdest things can happen */
+	if (y2 > PALETTE_HEIGHT)
+	{
+		gdk_draw_rectangle(widget->window, widget->style->black_gc,
+			TRUE, event->area.x, PALETTE_HEIGHT,
+			event->area.width, y2 - PALETTE_HEIGHT);
+		if (event->area.y >= PALETTE_HEIGHT) return (TRUE);
+		y2 = PALETTE_HEIGHT;
+	}
+	if (x2 > PALETTE_WIDTH)
+	{
+		gdk_draw_rectangle(widget->window, widget->style->black_gc,
+			TRUE, PALETTE_WIDTH, event->area.y,
+			x2 - PALETTE_WIDTH, y2);
+		if (event->area.x >= PALETTE_WIDTH) return (TRUE);
+		x2 = PALETTE_WIDTH;
+	}
+
+	gdk_draw_rgb_image(widget->window, widget->style->black_gc,
+		event->area.x, event->area.y,
+		x2 - event->area.x, y2 - event->area.y,
+		GDK_RGB_DITHER_NONE, mem_pals + event->area.y * PALETTE_W3 +
+		event->area.x * 3, PALETTE_W3);
+
+	return (TRUE);
 }
 
 
-static gint motion_palette( GtkWidget *widget, GdkEventMotion *event )
+static gboolean motion_palette(GtkWidget *widget, GdkEventMotion *event, gpointer user_data)
 {
 	GdkModifierType state;
-	int x, y, px, py, pindex;
-
-	px = event->x;
-	py = event->y;
-	pindex = (py-39+34)/16;
-
-	mtMAX(pindex, pindex, 0)
-	mtMIN(pindex, pindex, mem_cols-1)
+	int x, y, pindex;
 
 	if (event->is_hint) gdk_window_get_pointer (event->window, &x, &y, &state);
 	else
@@ -994,14 +1001,17 @@ static gint motion_palette( GtkWidget *widget, GdkEventMotion *event )
 		state = event->state;
 	}
 
-	if ( drag_index && drag_index_vals[1] != pindex )
+	pindex = (y - PALETTE_SWATCH_Y) / PALETTE_SWATCH_H;
+	pindex = pindex < 0 ? 0 : pindex >= mem_cols ? mem_cols - 1 : pindex;
+
+	if (drag_index && (drag_index_vals[1] != pindex))
 	{
-		mem_pal_index_move( drag_index_vals[1], pindex );
+		mem_pal_index_move(drag_index_vals[1], pindex);
 		init_pal();
 		drag_index_vals[1] = pindex;
 	}
 
-	return TRUE;
+	return (TRUE);
 }
 
 static gint release_palette( GtkWidget *widget, GdkEventButton *event )
@@ -1019,6 +1029,7 @@ static gint release_palette( GtkWidget *widget, GdkEventButton *event )
 			if ( mem_img_bpp == 1 )
 				mem_canvas_index_move( drag_index_vals[0], drag_index_vals[1] );
 
+			mem_undo_prepare();
 			canvas_undo_chores();
 		}
 	}
@@ -1030,63 +1041,65 @@ static gint click_palette( GtkWidget *widget, GdkEventButton *event )
 {
 	int px, py, pindex;
 
+
+	/* Filter out multiple clicks */
+	if (event->type != GDK_BUTTON_PRESS) return (TRUE);
+
 	px = event->x;
 	py = event->y;
-	pindex = (py-2)/16;
-	if (py < 2) pindex = -1;
+	if (py < PALETTE_SWATCH_Y) return (TRUE);
+	pindex = (py - PALETTE_SWATCH_Y) / PALETTE_SWATCH_H;
+	if (pindex >= mem_cols) return (TRUE);
 
-	if (pindex>=0 && pindex<mem_cols)
+	if (px < PALETTE_SWATCH_X) colour_selector(COLSEL_EDIT_ALL + pindex);
+	else if (px < PALETTE_CROSS_X)		// Colour A or B changed
 	{
-		if ( px<22 ) pressed_allcol( NULL, NULL );
-		if ( px>22 && px <53 )		// Colour A or B changed
+		if (event->button == 1)
 		{
-			if ( event->button == 1 )
-			{
-				if ( (event->state & GDK_CONTROL_MASK) )
-				{
-					mem_col_B = pindex;
-					mem_col_B24 = mem_pal[mem_col_B];
-				}
-				else if ( (event->state & GDK_SHIFT_MASK) )
-				{
-					mem_pal_copy( brcosa_pal, mem_pal );
-					drag_index = TRUE;
-					drag_index_vals[0] = pindex;
-					drag_index_vals[1] = pindex;
-					gdk_window_set_cursor( drawing_palette->window, move_cursor );
-				}
-				else
-				{
-					mem_col_A = pindex;
-					mem_col_A24 = mem_pal[mem_col_A];
-				}
-			}
-			if ( event->button == 3 )
+			if (event->state & GDK_CONTROL_MASK)
 			{
 				mem_col_B = pindex;
 				mem_col_B24 = mem_pal[mem_col_B];
 			}
-
-			repaint_top_swatch();
-			init_pal();
-			gtk_widget_queue_draw( drawing_col_prev );	// Update widget
+			else if (event->state & GDK_SHIFT_MASK)
+			{
+				mem_pal_copy(brcosa_pal, mem_pal);
+				drag_index = TRUE;
+				drag_index_vals[0] = pindex;
+				drag_index_vals[1] = pindex;
+				gdk_window_set_cursor(drawing_palette->window, move_cursor);
+			}
+			else
+			{
+				mem_col_A = pindex;
+				mem_col_A24 = mem_pal[mem_col_A];
+			}
 		}
-		if ( px>=53 )			// Mask changed
+		else if (event->button == 3)
 		{
-			mem_prot_mask[pindex] ^= 255;
-
-			repaint_swatch(pindex);				// Update swatch
-			gtk_widget_queue_draw_area( widget, 0, event->y-16,
-				PALETTE_WIDTH, 32 );			// Update widget
-
-			mem_mask_init();		// Prepare RGB masks
-			/* !!! Do the same for any other kind of preview */
-			if ((tool_type == TOOL_SELECT) && (marq_status >= MARQUEE_PASTE))
-				update_all_views();
+			mem_col_B = pindex;
+			mem_col_B24 = mem_pal[mem_col_B];
 		}
+
+		init_pal();
+	}
+	else /* if (px >= PALETTE_CROSS_X) */		// Mask changed
+	{
+		mem_prot_mask[pindex] ^= 255;
+
+		/* Update & redraw swatch */
+		repaint_swatch(pindex);
+		gtk_widget_queue_draw_area(widget,
+			PALETTE_CROSS_X, PALETTE_SWATCH_Y + PALETTE_SWATCH_H * pindex,
+			PALETTE_WIDTH - PALETTE_CROSS_X, PALETTE_SWATCH_H);
+
+		mem_mask_init();		// Prepare RGB masks
+		/* !!! Do the same for any other kind of preview */
+		if ((tool_type == TOOL_SELECT) && (marq_status >= MARQUEE_PASTE))
+			update_all_views();
 	}
 
-	return TRUE;
+	return (TRUE);
 }
 
 void toolbar_palette_init(GtkWidget *box)		// Set up the palette area
@@ -1094,54 +1107,40 @@ void toolbar_palette_init(GtkWidget *box)		// Set up the palette area
 	GtkWidget *vbox, *hbox, *scrolledwindow_palette, *viewport_palette;
 
 
-	vbox = gtk_vbox_new (FALSE, 0);
-	if ( toolbar_status[TOOLBAR_PALETTE] ) gtk_widget_show (vbox);
-	gtk_box_pack_start (GTK_BOX (box), vbox, FALSE, TRUE, 0);
+	toolbar_boxes[TOOLBAR_PALETTE] = vbox = pack(box, gtk_vbox_new(FALSE, 0));
+	if (toolbar_status[TOOLBAR_PALETTE]) gtk_widget_show(vbox);
 
-	toolbar_boxes[TOOLBAR_PALETTE] = vbox;		// Hide palette area
+	hbox = pack5(vbox, gtk_hbox_new(FALSE, 0));
+	gtk_widget_show(hbox);
 
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 5);
+	drawing_col_prev = gtk_drawing_area_new();
+	gtk_widget_show(drawing_col_prev);
+	gtk_widget_set_usize(drawing_col_prev, PREVIEW_WIDTH, PREVIEW_HEIGHT);
 
-
-	drawing_col_prev = gtk_drawing_area_new ();
-#if GTK_MAJOR_VERSION == 2
-	viewport_palette = gtk_viewport_new (NULL, NULL);
-	gtk_widget_show (viewport_palette);
-	gtk_viewport_set_shadow_type( GTK_VIEWPORT(viewport_palette), GTK_SHADOW_IN );
-	gtk_box_pack_start( GTK_BOX(hbox), viewport_palette, TRUE, FALSE, 0 );
-
-	gtk_container_add (GTK_CONTAINER (viewport_palette), drawing_col_prev);
-#endif
-#if GTK_MAJOR_VERSION == 1
-	gtk_box_pack_start( GTK_BOX(hbox), drawing_col_prev, TRUE, FALSE, 0 );
-#endif
-	gtk_widget_set_usize( drawing_col_prev, PREVIEW_WIDTH, PREVIEW_HEIGHT );
-
-	gtk_widget_show( drawing_col_prev );
 	gtk_signal_connect_object( GTK_OBJECT(drawing_col_prev), "button_release_event",
 		GTK_SIGNAL_FUNC (click_colours), GTK_OBJECT(drawing_col_prev) );
 	gtk_signal_connect_object( GTK_OBJECT(drawing_col_prev), "expose_event",
 		GTK_SIGNAL_FUNC (expose_preview), GTK_OBJECT(drawing_col_prev) );
 	gtk_widget_set_events (drawing_col_prev, GDK_ALL_EVENTS_MASK);
 
-	scrolledwindow_palette = gtk_scrolled_window_new (NULL, NULL);
+	viewport_palette = gtk_viewport_new (NULL, NULL);
+	gtk_widget_show (viewport_palette);
+	gtk_viewport_set_shadow_type( GTK_VIEWPORT(viewport_palette), GTK_SHADOW_IN );
+	gtk_container_add (GTK_CONTAINER (viewport_palette), drawing_col_prev);
+	gtk_box_pack_start( GTK_BOX(hbox), viewport_palette, TRUE, FALSE, 0 );
+
+	scrolledwindow_palette = xpack(vbox, gtk_scrolled_window_new(NULL, NULL));
 	gtk_widget_show (scrolledwindow_palette);
-	gtk_box_pack_start (GTK_BOX (vbox), scrolledwindow_palette, TRUE, TRUE, 0);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_palette),
 		GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
 
-	viewport_palette = gtk_viewport_new (NULL, NULL);
+	drawing_palette = gtk_drawing_area_new();
+	gtk_widget_show(drawing_palette);
+	gtk_widget_set_usize(drawing_palette, PALETTE_WIDTH, 64);
+	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolledwindow_palette),
+		drawing_palette);
+	fix_scroll(scrolledwindow_palette);
 
-	gtk_widget_set_usize( viewport_palette, PALETTE_WIDTH, 64 );
-	gtk_widget_show (viewport_palette);
-	gtk_container_add (GTK_CONTAINER (scrolledwindow_palette), viewport_palette);
-
-	drawing_palette = gtk_drawing_area_new ();
-	gtk_widget_set_usize( drawing_palette, PALETTE_WIDTH, 64 );
-	gtk_container_add (GTK_CONTAINER (viewport_palette), drawing_palette);
-	gtk_widget_show( drawing_palette );
 	gtk_signal_connect_object( GTK_OBJECT(drawing_palette), "expose_event",
 		GTK_SIGNAL_FUNC (expose_palette), GTK_OBJECT(drawing_palette) );
 	gtk_signal_connect_object( GTK_OBJECT(drawing_palette), "button_press_event",
@@ -1215,12 +1214,6 @@ void pressed_toolbar_toggle( GtkMenuItem *menu_item, gpointer user_data, gint it
 ///	PATTERNS/TOOL PREVIEW AREA
 
 
-void toolbar_preview_init()		// Initialize memory for preview area
-{
-	mem_prev = grab_memory( 3*PREVIEW_WIDTH*PREVIEW_HEIGHT, 0 );
-}
-
-
 void mem_set_brush(int val)			// Set brush, update size/flow/preview
 {
 	int offset, i, j, k, o, o2;
@@ -1245,24 +1238,83 @@ void mem_set_brush(int val)			// Set brush, update size/flow/preview
 	if ( drawing_col_prev ) gtk_widget_queue_draw( drawing_col_prev );
 }
 
+#include "graphics/xbm_patterns.xbm"
+#if (PATTERN_GRID_W * 8 != xbm_patterns_width) || (PATTERN_GRID_H * 8 != xbm_patterns_height)
+#error "Mismatched patterns bitmap"
+#endif
+
+/* Create RGB dump of patterns to display, with each pattern repeated 4x4 */
+unsigned char *render_patterns()
+{
+	png_color *p;
+	unsigned char *buf, *dest;
+	int i = 0, j, x, y, h, b;
+
+#define PAT_ROW_L (PATTERN_GRID_W * (8 * 4 + 4) * 3)
+#define PAT_8ROW_L (8 * PAT_ROW_L)
+	buf = calloc(1, PATTERN_GRID_H * (8 * 4 + 4) * PAT_ROW_L);
+	dest = buf + 2 * PAT_ROW_L + 2 * 3;
+	for (y = 0; y < PATTERN_GRID_H; y++ , dest += (8 * 3 + 4) * PAT_ROW_L)
+	{
+		for (h = 0; h < 8; h++)
+		for (x = 0; x < PATTERN_GRID_W; x++ , dest += (8 * 3 + 4) * 3)
+		{
+			b = xbm_patterns_bits[i++];
+			for (j = 0; j < 8; j++ , b >>= 1)
+			{
+				p = mem_col_24 + (b & 1);
+				*dest++ = p->red;
+				*dest++ = p->green;
+				*dest++ = p->blue;
+			}
+			memcpy(dest, dest - 8 * 3, 8 * 3);
+			memcpy(dest + 8 * 3, dest - 8 * 3, 2 * 8 * 3);
+		}
+		memcpy(dest, dest - PAT_8ROW_L, PAT_8ROW_L);
+		memcpy(dest + PAT_8ROW_L, dest - PAT_8ROW_L, 2 * PAT_8ROW_L);
+	}
+#undef PAT_8ROW_L
+#undef PAT_ROW_L
+	return (buf);
+}
+
+/* Set 0-1 indexed image as new patterns */
+void set_patterns(unsigned char *src)
+{
+	int i, j, b, l = PATTERN_GRID_W * PATTERN_GRID_H * 8;
+
+	for (i = 0; i < l; i++)
+	{
+		for (b = j = 0; j < 8; j++) b += *src++ << j;
+		xbm_patterns_bits[i] = b;
+	}
+}
+
 void mem_pat_update()			// Update indexed and then RGB pattern preview
 {
-	int i, j, k, l;
+	int i, j, k, l, ii, b;
 
 	if ( mem_img_bpp == 1 )
 	{
 		mem_col_A24 = mem_pal[mem_col_A];
 		mem_col_B24 = mem_pal[mem_col_B];
 	}
-	mem_pattern = (unsigned char *)mem_patterns[tool_pat];
 
-	for (i = 0; i < 8 * 8; i++)
+	/* Pattern bitmap starts here */
+	l = mem_tool_pat * 8 - (mem_tool_pat % PATTERN_GRID_W) * 7;
+
+	/* Set up pattern maps from XBM */
+	for (i = ii = 0; i < 8; i++)
 	{
-		j = mem_pattern[i] ^ 1;
-		mem_col_pat[i] = mem_col_[j];
-		mem_col_pat24[i * 3 + 0] = mem_col_24[j].red;
-		mem_col_pat24[i * 3 + 1] = mem_col_24[j].green;
-		mem_col_pat24[i * 3 + 2] = mem_col_24[j].blue;
+		b = xbm_patterns_bits[l + i * PATTERN_GRID_W];
+		for (k = 0; k < 8; k++ , ii++ , b >>= 1)
+		{
+			mem_pattern[ii] = j = b & 1;
+			mem_col_pat[ii] = mem_col_[j];
+			mem_col_pat24[ii * 3 + 0] = mem_col_24[j].red;
+			mem_col_pat24[ii * 3 + 1] = mem_col_24[j].green;
+			mem_col_pat24[ii * 3 + 2] = mem_col_24[j].blue;
+		}
 	}
 
 	k = PREVIEW_WIDTH * 32 * 3;
@@ -1282,37 +1334,32 @@ void mem_pat_update()			// Update indexed and then RGB pattern preview
 		gtk_widget_queue_draw(drawing_canvas);
 }
 
-void repaint_top_swatch()			// Update selected colours A & B
+void update_top_swatch()			// Update selected colours A & B
 {
-	int i, j, r[2], g[2], b[2], nx, ny;
+	unsigned char AA[3], BB[3], *tmp;
+	int i, j;
 
-	if ( mem_img_bpp == 1 )
-	{
-		mem_col_A24 = mem_pal[mem_col_A];
-		mem_col_B24 = mem_pal[mem_col_B];
-	}
-	r[0] = mem_col_A24.red;
-	g[0] = mem_col_A24.green;
-	b[0] = mem_col_A24.blue;
-	r[1] = mem_col_B24.red;
-	g[1] = mem_col_B24.green;
-	b[1] = mem_col_B24.blue;
+	AA[0] = mem_col_A24.red;
+	AA[1] = mem_col_A24.green;
+	AA[2] = mem_col_A24.blue;
+	BB[0] = mem_col_B24.red;
+	BB[1] = mem_col_B24.green;
+	BB[2] = mem_col_B24.blue;
 
-	for ( j=0; j<20; j++ )
+#define dAB (10 * PREVIEW_WIDTH * 3 + 10 * 3)
+	for (j = 1; j <= 20; j++)
 	{
-		for ( i=0; i<20; i++ )
+		tmp = (mem_prev + 1 * 3) + j * (PREVIEW_WIDTH * 3);
+		for (i = 0; i < 20; i++ , tmp += 3)
 		{
-			nx = i+1; ny = j+1;
-			mem_prev[ 0 + 3*( nx + ny*PREVIEW_WIDTH) ] = r[0];
-			mem_prev[ 1 + 3*( nx + ny*PREVIEW_WIDTH) ] = g[0];
-			mem_prev[ 2 + 3*( nx + ny*PREVIEW_WIDTH) ] = b[0];
+			tmp[0] = AA[0];
+			tmp[1] = AA[1];
+			tmp[2] = AA[2];
 
-			nx = i+11; ny = j+11;
-			mem_prev[ 0 + 3*( nx + ny*PREVIEW_WIDTH) ] = r[1];
-			mem_prev[ 1 + 3*( nx + ny*PREVIEW_WIDTH) ] = g[1];
-			mem_prev[ 2 + 3*( nx + ny*PREVIEW_WIDTH) ] = b[1];
+			tmp[dAB + 0] = BB[0];
+			tmp[dAB + 1] = BB[1];
+			tmp[dAB + 2] = BB[2];
 		}
 	}
-
-	if ( drawing_col_prev ) gtk_widget_queue_draw( drawing_col_prev );
+#undef dAB
 }
